#! /bin/ksh

#
# @(#) Script de mise-a-jour de HPU
# 

# 
# Utilisation ksh pour comparaison de chaines
# 
# 
# 
# 
# La trace est redirigee dans un fichier (DisplayTrace, resultat de tar, ...)
# 
# 

# 


### DEBUG ###
# set -x
set -u
### FIN DEBUG ###



##########################################################################
# DisplayTrace
###########################################################################

# 
# Affichage message de trace (sur sortie standard)
# 
# 
# Argument 1 : texte du message
# 
# 

DisplayTrace ()
{
    # RETURN_CODE=0
    
    echo "$1" >> ${FILE_TRACE}
    
    # return ${RETURN_CODE}
}


##########################################################################
# GetINIValue
###########################################################################

# 
# Recuperation de la valeur d'une variable, dans une section d'un fichier .INI
# 
# Argument 1 : nom de fichier .INI
# Argument 2 : Nom de la variable
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection.
# 
# Normaliser la valeur retournee en cas d'erreur (chaine TRES specifique) 
# Si on choisit de retourner la chaine vide, pas de difference avec une valeur
# volontaire de "chaine vide".
#

GetINIValue ()
{
    
    
    # Pas de trace car redirection exploitee !
    # DisplayTrace ">>> GetINIValue"
    

    
    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        FILE="$1"
        IDENTIFIER="$2"
    
        LINE=`grep '^'"${IDENTIFIER}=" "${FILE}"`
        if [ $? -eq 0 ]
        then
            echo "${LINE}" | cut -d'=' -f2
        else
            echo '\07Error'
        
        fi
    
    fi
    
    RETURN_CODE=1
}


##########################################################################
# GetUserID
###########################################################################

# 
# Obtention du numero d'utilisateur courant (UID)
# 
# Aucun argument 
# 
# 
# 
#

GetUserID ()
{
    RETURN_CODE=0
    

    # DisplayTrace ""
    # DisplayTrace ">>> GetUserID"


    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
    
        TEMP_USER_ID=`LANG=C ${ID} | ${CUT} -d'=' -f2 | ${CUT} -d'(' -f1`
        if [ $? -eq 0 ]
        then
            echo "${TEMP_USER_ID}"
        else
            echo '\07Error'
        fi
        
        # DisplayTrace "    TEMP_USER_ID : \"${TEMP_USER_ID}\"" 

    fi    


    return ${RETURN_CODE}
}

##########################################################################
# InitCommandsVariables
###########################################################################

# 
# Initialisation des variables correspondant commandes systeme
# 
# 
# Mettre a jour la fonction CheckEnvironment () si de nouvelels commandes sont 
# ajoutees.
#

InitCommandsVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitCommandsVariables"
    
       
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        AWK=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "awk"` 
        RETURN_CODE=$?

        DisplayTrace "    AWK : ${AWK}"
    fi
        
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CAT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "cat"` 
        RETURN_CODE=$?

        DisplayTrace "    CAT : ${CAT}"
    fi
    

    if [ ${RETURN_CODE} -eq 0 ]
    then    
        CHGRP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "chgrp"` 
        RETURN_CODE=$?

        DisplayTrace "    CHGRP : ${CHGRP}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CHMOD=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "chmod"` 
        RETURN_CODE=$?

        DisplayTrace "    CHMOD : ${CHMOD}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CHOWN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "chown"` 
        RETURN_CODE=$?

        DisplayTrace "    CHOWN : ${CHOWN}"
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        CP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "cp"` 
        RETURN_CODE=$?

        DisplayTrace "    CP : ${CP}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CUT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "cut"` 
        RETURN_CODE=$?

        DisplayTrace "    CUT : ${CUT}"
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        EXPR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "expr"` 
        RETURN_CODE=$?

        DisplayTrace "    EXPR : ${EXPR}"
    fi
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        FIND=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "find"` 
        RETURN_CODE=$?

        DisplayTrace "    FIND : ${FIND}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        GREP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "grep"` 
        RETURN_CODE=$?

        DisplayTrace "    GREP : ${GREP}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        ID=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "id"` 
        RETURN_CODE=$?

        DisplayTrace "    ID : ${ID}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        KILL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "kill"` 
        RETURN_CODE=$?

        DisplayTrace "    KILL : ${KILL}"
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        LN=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "ln"` 
        RETURN_CODE=$?

        DisplayTrace "    LN : ${LN}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        LS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "ls"` 
        RETURN_CODE=$?

        DisplayTrace "    LS : ${LS}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        MKDIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "mkdir"` 
        RETURN_CODE=$?

        DisplayTrace "    MKDIR : ${MKDIR}"
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        MV=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "mv"` 
        RETURN_CODE=$?

        DisplayTrace "    MV : ${MV}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        PRINTF=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "printf"` 
        RETURN_CODE=$?

        DisplayTrace "    PRINTF : ${PRINTF}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        PS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "ps"` 
        RETURN_CODE=$?

        DisplayTrace "    PS : ${PS}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        RM=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "rm"` 
        RETURN_CODE=$?

        DisplayTrace "    RM : ${RM}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        RMDIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "rmdir"` 
        RETURN_CODE=$?

        DisplayTrace "    RMDIR : ${RMDIR}"
    fi

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        SED=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "sed"` 
        RETURN_CODE=$?

        DisplayTrace "    SED : ${SED}"
    fi
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        SORT=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "sort"` 
        RETURN_CODE=$?

        DisplayTrace "    SORT : ${SORT}"
    fi

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        TAR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "tar"` 
        RETURN_CODE=$?

        DisplayTrace "    TAR : ${TAR}"
    fi
    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        TR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "tr"` 
        RETURN_CODE=$?

        DisplayTrace "    TR : ${TR}"
    fi
    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        WC=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "wc"` 
        RETURN_CODE=$?

        DisplayTrace "    WC : ${WC}"
    fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        XARGS=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}commands" "xargs"` 
        RETURN_CODE=$?

        DisplayTrace "    XARGS : ${XARGS}"
    fi

    
    return ${RETURN_CODE}
}


##########################################################################
# InitActionFilesVariables
###########################################################################

# 
# Initialisation des variables correspondant aux fichiers des actions
# 
# Etiquette de la section : "action_files"
#
#
#

InitActionFilesVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitActionFilesVariables"

    
    
    # Fichiers des actions pour les differents features (unload, network, plugin) 
    # lors de l'installation et la desinstallation  
        
        
    FILE_ACTIONS_INSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_files" "file_actions_install"` 

    DisplayTrace "    FILE_ACTIONS_INSTALL : ${FILE_ACTIONS_INSTALL}"


    FILE_ACTIONS_UNINSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_files" "file_actions_uninstall"` 

    DisplayTrace "    FILE_ACTIONS_UNINSTALL : ${FILE_ACTIONS_UNINSTALL}"




     

    
    
    # Fichiers prologue et epilogue pour l'installer
    # A faire : penser aux chemins complets !!!
    
    FILE_ACTIONS_PROLOG_INSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_installers_files" "file_actions_prolog_install"` 

    DisplayTrace "    FILE_ACTIONS_PROLOG_INSTALL : ${FILE_ACTIONS_PROLOG_INSTALL}"


    FILE_ACTIONS_EPILOG_INSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_installers_files" "file_actions_epilog_install"` 

    DisplayTrace "    FILE_ACTIONS_EPILOG_INSTALL : ${FILE_ACTIONS_EPILOG_INSTALL}"



    # Fichiers prologue et epilogue pour l'uninstaller
    # A faire : penser aux chemins complets !!!
    
    FILE_ACTIONS_PROLOG_UNINSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_installers_files" "file_actions_prolog_uninstall"` 

    DisplayTrace "    FILE_ACTIONS_PROLOG_UNINSTALL : ${FILE_ACTIONS_PROLOG_UNINSTALL}"

    
    FILE_ACTIONS_EPILOG_UNINSTALL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}action_installers_files" "file_actions_epilog_uninstall"` 

    DisplayTrace "    FILE_ACTIONS_EPILOG_UNINSTALL : ${FILE_ACTIONS_EPILOG_UNINSTALL}"

    

    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckReadableFile ${FILE_ACTIONS_PROLOG_INSTALL}
        RETURN_CODE=$?
    fi    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckReadableFile ${FILE_ACTIONS_EPILOG_INSTALL}
        RETURN_CODE=$?
    fi    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckReadableFile ${FILE_ACTIONS_PROLOG_UNINSTALL}
        RETURN_CODE=$?
    fi    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckReadableFile ${FILE_ACTIONS_EPILOG_UNINSTALL}
        RETURN_CODE=$?
    fi    
    
    return ${RETURN_CODE}
}




##########################################################################
# InitArchiveFileVariable
###########################################################################

# 
# Initialisation des variables correspondant aux fichiers archives (tar)
# 
# Etiquette de la section : "archive_files"
#
# Uniquement si on est en mode installation
# Necessite la liste des features a installer (cree par PrepareActionsLists ())
#

InitArchiveFileVariable ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitArchiveFileVariable"

    
    
    if [ "${CONTEXT}" = "inst" ]
    then
        FILE_ARCHIVE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}archive_files" "file_archives"` 

        DisplayTrace "    FILE_ARCHIVE : ${FILE_ARCHIVE}"

        if [ ${RETURN_CODE} -eq 0 ]
        then

            CheckReadableFile ${FILE}
            RETURN_CODE=$?

        fi    
    fi    
        
    return ${RETURN_CODE}
}



##########################################################################
# InitSystemFilesVariables
###########################################################################

# 
# Initialisation des variables correspondant aux fichiers systeme
# 
# 
# Etiquette de la section : "system_files"
#
#
#

InitSystemFilesVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitSystemFilesVariables"

    
        
    FILE_SERVICES=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_services"` 

    DisplayTrace "    FILE_SERVICES : ${FILE_SERVICES}"


    FILE_PROFILE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_profile"` 

    DisplayTrace "    FILE_PROFILE : ${FILE_PROFILE}"


    FILE_STARTUP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_startup"` 

    DisplayTrace "    FILE_STARTUP : ${FILE_STARTUP}"

    
    FILE_PASSWD=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_passwd"` 

    DisplayTrace "    FILE_PASSWD : ${FILE_PASSWD}"


    FILE_GROUP=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_group"` 

    DisplayTrace "    FILE_GROUP : ${FILE_GROUP}"


    FILE_INET_SERVICES=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_files" "file_inet_services"` 

    DisplayTrace "    FILE_INET_SERVICES : ${FILE_INET_SERVICES}"

    
    
    return ${RETURN_CODE}
}






##########################################################################
# InitOtherSystemVariables
###########################################################################

#
# Initialisation des variables correspondant des informations systeme diverses
#
#
# Etiquette de la section : "system_other"
#
# 
#
#

InitOtherSystemVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitOtherSystemVariables"

    
        
    DAEMON_INETD=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_other" "daemon_inetd"` 

    DisplayTrace "    DAEMON_INETD : ${DAEMON_INETD}"


    DAEMON_INETD_RECONFIG_SIGNAL=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}system_other" "daemon_inetd_reconfig_signal"` 

    DisplayTrace "    DAEMON_INETD_RECONFIG_SIGNAL : ${DAEMON_INETD_RECONFIG_SIGNAL}"

    
    return ${RETURN_CODE}
}


##########################################################################
# PrepareINIFile
###########################################################################

# Preparation du fichier .INI (par decomposition) en vue de son exploitation
# 
# Argument 1 : nom de fichier
# Argument 2 : prefixe de nom fichier pour les fichiers generes par decomposition
#
#
# Les commandes (csplit, mv, ...) sont appeles avec leur vrai nom et pas via 
# une variable car on n'a pas encore lu le fichier de configuration qui 
# contient les variables.
#
# Attention : on suppose qu'il n'y a pas plus de 99 sections dans le fichier
# .INI.
#

PrepareINIFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> PrepareINIFile"

    
    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=2
    else    

        # PREFIX_TMP est le prefixe des fichiers temporaires generes par csplit.
        # Ils sont ensuite renommes avec PREFIX et le nom de la section qu'il contiennent
        FILE="$1"
        PREFIX_TEMP="tmp_$1_"
        PREFIX="$2"
        
        if [ ! -f "${FILE}" ]
        then
            echo "Error file not found \"${FILE}\"." 1>&2
            RETURN_CODE=3      
        else
            
            \csplit -ks -f ${PREFIX_TEMP} ${FILE} '/^\[.*\]/' '{100}' 2>/dev/null
            RETURN_CODE=$?
            
            # Deplacement des fichiers generes dans le repertoire temporaire
            \mv ${PREFIX_TEMP}?? ${DIR_TEMP}
            RETURN_CODE=$?
            
              
            if [ ${RETURN_CODE} -eq 0 ]
            then
                for NOM in ${DIR_TEMP}/${PREFIX_TEMP}??
                do


                    # DisplayTrace ${NOM}


                    LINE=`\grep '^\[.*\]' ${NOM} | \tr -d ' '`
                    if [ ! -z "${LINE}" ]
                    then

                        # DisplayTrace "    LINE : \"${LINE}\"" 


                        SECTION=`\expr "${LINE}" ':' '\[\(.*\)\]'`

                        DisplayTrace "    Section found : \"${SECTION}\"" 

                        if [ -f "${DIR_TEMP}/${PREFIX}${SECTION}" ]
                        then
                            # Erreur fatale : une section est presente deux fois dans le fichier
                            echo "Section file \"${PREFIX}${SECTION}\" already existing !" 1>&2
                            RETURN_CODE=1
                            break
                        else
                            \mv "${NOM}" "${DIR_TEMP}/${PREFIX}${SECTION}"
                        fi    
                    else
                        # Declarations de variables hors de toute section (en debut de fichier)
                        # Normalement il n'y en pas
                        \mv "${NOM}" "${DIR_TEMP}/${PREFIX}global"
                    fi    

                done
            else
                echo "Error moving files to directory \"${DIR_TEMP}\" " 1>&2
                RETURN_CODE=1
            fi
        fi
    
    fi
    
    return ${RETURN_CODE}
}




##########################################################################
# CheckExistingDirectory
###########################################################################

# Verification si repertoire existant
#
# Pas de vrification si droits en ecriture !
#

CheckExistingDirectory ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckExistingDirectory"

    
    if [ ! -d $1 ]
    then
        echo "Error checking existing directory \"$1\"." 1>&2
        RETURN_CODE=1
    
    fi
    
    return ${RETURN_CODE}
}


###########################################################################
# CheckExecutableFile
###########################################################################

# Verification si fichier executable
#

CheckExecutableFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckExecutableFile"

    
    if [ ! -x $1 ]
    then
        echo "Error checking executable file \"$1\"." 1>&2
        RETURN_CODE=1
    
    fi
    
    return ${RETURN_CODE}
}




##########################################################################
# CheckReadableFile
###########################################################################

# Verification si fichier existant et lisible
#

CheckReadableFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckReadableFile"

    
    if [ ! -r $1 ]
    then
        echo "Error checking readable file \"$1\"." 1>&2
        RETURN_CODE=1
    
    fi
    
    return ${RETURN_CODE}
}


##########################################################################
# CheckWriteableFile
###########################################################################

#
# Verification si fichier modifiable
#

CheckWriteableFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckWriteableFile"

    
    if [ ! -w $1 ]
    then
        echo "Error checking writeable file \"$1\"." 1>&2
        RETURN_CODE=1
    
    fi
    
    return ${RETURN_CODE}
}


##########################################################################
# FormatPlatformName
###########################################################################

# 
# Reformatage du nom de plateforme transmis par InstallShield
# 
# 
# Argument 1 : nom de plateforme
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# Aucun affichage parasite doit etre fait dans cette fonction.
# 
# Pour Solaris, le nom peut varier entre "Solaris" et "SunOS" selon la version du 
# JVM utilise.
# 
# Pour Linux, differenciation selon l'architecture pour "linux" et zlinux"
# 
# 

FormatPlatformName ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> FormatPlatformName"
    

    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        NAME=`\echo $1 | tr '[A-Z]' '[a-z]'`
        case "${NAME}" in
        
        "linux")

            case "${ARCHITECTURE}" in
            
            "x86"|"i386")
                echo "linux"
                ;;
            
            "s390") 
                echo "zlinux"
                ;;

            *) 
                echo '\07Error'
                RETURN_CODE=1  
                ;;       
            esac         

            ;;

        "solaris"|"sunos")
            echo "sun"
            ;;
            
        "aix")
            echo "aix"
            ;;
            
        "hp-ux")
            echo "hp"
            ;;
            
        *)
            echo '\07Error'
            RETURN_CODE=1        
            ;;
       esac
        
    fi    
    
    return ${RETURN_CODE}
}


##########################################################################
# UnformatPlatformName
###########################################################################

# 
# Reformatage du nom de plateforme intenre en un nom plus explicite
# Operation inverse de FormatPlatformName ()
# 
# Argument 1 : nom de plateforme
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# Aucun affichage parasite doit etre fait dans cette fonction.
# 
# On ne tient pas compte pour l'instant des architectures "zlinux".
# 
# 

UnformatPlatformName ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> UnformatPlatformName"
    

    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        NAME=`\echo $1 | tr '[A-Z]' '[a-z]'`
        case "${NAME}" in
        
        "linux")
            echo "Linux"
            ;;

        "sun")
            echo "SunOs"
            ;;
            
        "aix")
            echo "AIX"
            ;;
            
        "hp")
            echo "HP-UX"
            ;;
            
        *)
            echo '\07Error'
            RETURN_CODE=1        
            ;;
       esac
        
    fi    
    
    return ${RETURN_CODE}
}



##########################################################################
# NumberElementsList
###########################################################################

# 
# Determination du nombre d'elements d'une liste
# 
# 
# Argument 1 : liste
# Argument 2 : separateur
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# 
# Tous les cas ne sont pas geres
# 
# 
# 

NumberElementsList ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> NumberElementsList"
    

    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        echo $1 | \awk -F"$2" '{print NF}'
        RETURN_CODE=$?
    fi    
    
    return ${RETURN_CODE}
}


##########################################################################
# GetFieldSublist
###########################################################################

# 
# Recuperation d'un element d'une sous-liste
# 
# 
# Argument 1 : liste
# Argument 2 : separateur liste
# Argument 3 : separateur sous-liste
# Argument 4 : clef
# Argument 5 : indice element
# 
# Si la recherche est infructueuse le code retour reste a 0 !!
# 
# Exemple : 
# - Separateur de liste : "|" 
# - Separateur de sous-liste : ":" 
# La clef est l'element 1 d'une sous-liste
#  
# clef1:elt2:elt3|clef2:elt2:elt3:elt4|clef3...
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# Aucun affichage parasite sur stdout ne doit etre fait dans cette fonction.
# 
# Tous les cas ne sont pas geres
# Il n'y a pas de controle sur l'indice de l'element
# 
# 

GetFieldSublist ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> GetFieldSublist"
    

    if [ $# -ne 5 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        echo "${1}" | \tr "$2" '\n' | \grep "^${4}${3}" | \cut -d"$3" -f"$5"
        RETURN_CODE=$?
    fi    
    
    return ${RETURN_CODE}
}


##########################################################################
# GetFieldList
###########################################################################

# 
# Recuperation d'un element d'une liste
# 
# 
# Argument 1 : liste
# Argument 2 : separateur liste
# Argument 3 : indice element
# 
# Si la recherche est infructueuse le code retour reste a 0 !!
# 
# 
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# Aucun affichage parasite sur stdout ne doit etre fait dans cette fonction.
#
# Tous les cas ne sont pas geres
# Il n'y a pas de controle sur l'indice de l'element
# 
# 
# 

GetFieldList ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> GetFieldList"
    

    if [ $# -ne 3 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        echo "${1}" | \cut -d"$2" -f"$3"
        RETURN_CODE=$?
    fi    
    
    return ${RETURN_CODE}
}



##########################################################################
# ReverseString
###########################################################################

# 
# Inversion des mots d'une chaine
# 
# Argument 1 : chaine a inverser (elle doit etre entre guillemets lors de 
#              l'appel)
# 
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection
# Aucun affichage parasite sur stdout ne doit etre fait dans cette fonction.
#

ReverseString ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> ReverseString"
    

    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else    
        RESULT_STRING=""    
        
        for WORD in ${1}
        do
            RESULT_STRING="${WORD} ${RESULT_STRING}"
        done
         
        echo "${RESULT_STRING}"
    fi    
    
    return ${RETURN_CODE}
}



##########################################################################
# ReverseFile
###########################################################################

# 
# Creation d'un nouveau fichier a partir d'un autre en inversant l'ordre de ses lignes
# 
# Argument 1 : nom de fichier en entree
# Argument 2 : nom de fichier genere
# 
#

ReverseFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ReverseFile"


    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    

        CheckReadableFile $1
        RETURN_CODE=$?

        if [ ${RETURN_CODE} -eq 0 ]
        then
    
            # Creation du fichier des actions de desinstallation : inversion
            # de l'ordre des lignes du fichier d'installation

            ${CAT} -n ${1} | ${SORT} -nr | ${CUT} -f2- > ${2}
            RETURN_CODE=$?
         
        fi 
    fi    
    
    return ${RETURN_CODE}
}


##########################################################################
# CheckEmptyDir
###########################################################################

# 
# Indication si un repertoire est vide
# 
# Argument 1 : repertoire
# 
# Retourne 0 si repertoire vide 
# Retourne 3 si repertoire non vide (ou inexistant)
#
# On considere que si le repertoire n'existe pas, il est non vide (car cette 
# fonction est souvent utilisee en vue de la suppression du repertoire)
#
# Actuellement la variable (globale) REURN_CODE est utilisee : attention aux 
# effets de bord avec l'appelant.
# 
#

CheckEmptyDir ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckEmptyDir"



    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
        if [ -d $1 ]
        then
            ${LS} -f $1 | ${AWK} 'NR >= 3 {exit 2}'
            if [ $? -eq 2 ]
            then
                RETURN_CODE=3
            else    
                RETURN_CODE=0
            fi
        else
            RETURN_CODE=3
        fi    
    fi
    
    
    return ${RETURN_CODE}
}


##########################################################################
# CheckUser
###########################################################################

# 
# Verification de l'existence d'un utilisateur
# 
# Argument 1 : nom d'utilisateur
# 
# 
# Utilisation de getent (1/1M) ? Commande inexistante sur HP-UX
#

CheckUser ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckUser"


    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
    

        DisplayTrace "    USER : \"$1\"" 

    
    
        ${GREP} '^'"$1:" ${FILE_PASSWD} >/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            RETURN_CODE=0
        else
            echo "User name \"$1\"not found" 1>&2
            RETURN_CODE=2
        fi
    fi    


    return ${RETURN_CODE}
}



##########################################################################
# CheckGroup
###########################################################################

# 
# Verification de l'existence d'un groupe
# 
# Argument 1 : nom de groupe
# 
# 
# Utilisation de getent (1/1M) ? Mais commande inexistante sur HP-UX
#

CheckGroup ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> CheckGroup"


    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
    
    

        DisplayTrace "    GROUP : \"$1\"" 

    
    
        ${GREP} '^'"$1:" ${FILE_GROUP} >/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            RETURN_CODE=0
        else
            echo "Group name \"$1\"not found" 1>&2
            RETURN_CODE=2
        fi
    fi    


    return ${RETURN_CODE}
}



##########################################################################
# SetPermissionsFile
###########################################################################

# 
# Modification des permissions d'un fichier
# 
# Argument 1 : permissions (notation octale)
# Argument 2 : nom de fichier
# 
# Controles supplementaires ? (existence fichier, notation octale ?)
#

SetPermissionsFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> SetPermissionsFile"

    
    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
        ### PERMISSIONS="$1"
        ### FILE="$2"
    
    

        DisplayTrace "    PERMISSIONS : \"$1\"" 
        DisplayTrace "    FILE        : \"$2\"" 

    
    
        ${CHMOD} $1 $2
        if [ $? -ne 0 ]
        then
            echo "Error setting permissions \"$1\" for file \"$2\"" 1>&2
            RETURN_CODE=1
        fi
    fi    


    return ${RETURN_CODE}
}



##########################################################################
# SetOwnerFile
###########################################################################

# 
# Modification du proprietaire d'un fichier
# 
# Argument 1 : proprietaire
# Argument 2 : nom de fichier
# 
# Controles supplementaires ? (existence fichier, ?)
#

SetOwnerFile ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> SetOwnerFile"

    
    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    


        DisplayTrace "    OWNER       : \"$1\"" 
        DisplayTrace "    FILE        : \"$2\"" 

    
        if CheckUser $1
        then    
        
            ${CHOWN} $1 $2
            if [ $? -ne 0 ]
            then
                echo "Error setting owner \"$1\" for file \"$2\"" 1>&2
                RETURN_CODE=1
            fi
        fi
    fi    


    return ${RETURN_CODE}
}



##########################################################################
# SetGroupFile
###########################################################################

# 
# Modification du groupe d'un fichier
# 
# Argument 1 : groupe
# Argument 2 : nom de fichier
# 
# Controles supplementaires ? (existence fichier, ?)
#

SetGroupFile ()
{
    RETURN_CODE=0
    
    
    echo ""
    DisplayTrace ">>> SetGroupFile"
    
    
    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
    
        
        echo "    GROUP       : \"$1\"" 
        echo "    FILE        : \"$2\"" 
        
    
        if CheckGroup $1
        then    
    
            ${CHGRP} $1 $2
            if [ $? -ne 0 ]
            then
                echo "Error setting group \"$1\" for file \"$2\"" 1>&2
                RETURN_CODE=1
            fi
        fi    
    fi    


    return ${RETURN_CODE}
}






##########################################################################
# EvaluateIdentifier
###########################################################################

# 
# Recuperation de la valeur (evaluation) d'un identificateur de variable 
# 
# Argument 1 : nom variable
# 
# 
# Pas de trace sur stdout car redirection exploitee !
# 
#

EvaluateIdentifier ()
{
    RETURN_CODE=0
    
    
    # Pas de trace car redirection exploitee !
    # echo ""
    # DisplayTrace ">>> EvaluateIdentifier"
    

    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
    else   
    
        # Seconde evaluation 
        VALUE=`\eval \echo '\$'"${1}"`

        
        # echo "VALUE After evaluation of action arguments : \"${VALUE}\"" 
        

        # Affichage en retour
        echo "${VALUE}"
        
    fi
    
    return ${RETURN_CODE}
     
}



##########################################################################
# GetINIValue
###########################################################################

# 
# Recuperation de la valeur d'une variable, dans une section d'un fichier .INI
# 
# Argument 1 : nom de fichier .INI
# Argument 2 : Nom de la variable
#
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection.
# 
# Normaliser la valeur retournee en cas d'erreur (chaine TRES specifique) 
# Si on choisit de retourner la chaine vide, pas de difference avec une valeur
# volontaire de "chaine vide".
#

GetINIValue ()
{
    
    
    # Pas de trace car redirection exploitee !
    # DisplayTrace ">>> GetINIValue"
    

    
    if [ $# -ne 2 ]
    then
        echo "Missing or too many arguments ($#)"
        echo '\07Error'
        RETURN_CODE=1
    else    
        FILE="$1"
        IDENTIFIER="$2"
    
        LINE=`grep '^'"${IDENTIFIER}=" "${FILE}"`
        if [ $? -eq 0 ]
        then
            echo "${LINE}" | cut -d'=' -f2
        else
            echo '\07Error'
        
        fi
    
    fi
    
    RETURN_CODE=1
}



##########################################################################
# PrepareActionsLists
###########################################################################

#
# Preparation des listes d'actions a executer
#
# Plus de features.
#

PrepareActionsLists ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> PrepareActionsLists"


    # Utilisation du fichier d'actions correspondant en fonction du contexte
    # >>> A faire : FILE_ACTIONS a passer en parametre ????
    
###     case "${CONTEXT}" in
### 
###     "inst")    
### 
###         LIST_FEATURES_INSTALL=""
###         
###         for NAME in ${LIST_FEATURES}
###         do
###             # Exploitation uniquement de la propriete InstallShield "Active" (champ 2 de la sous-liste)
###             PROPERTY_INSTALL=`GetFieldSublist ${FEATURES} '|' ':' ${NAME} 2`
###             
### 
###             DisplayTrace "    Feature \"$NAME\" "
###             DisplayTrace "    PROPERTY_INSTALL     : \"${PROPERTY_INSTALL}\" "
### 
###         
###             if [ "${PROPERTY_INSTALL}" = "true" ]
###             then
###                 LIST_FEATURES_INSTALL="${LIST_FEATURES_INSTALL}${NAME} "
###             fi
###         
###         done
###         
### 
###         DisplayTrace "    List of features to install :"
###         DisplayTrace "    LIST_FEATURES_INSTALL : \"${LIST_FEATURES_INSTALL}\" "
### 
###         ;;
### 
### 
###     "uninst")    
### 
###         LIST_FEATURES_UNINSTALL=""
###         
###         for NAME in ${LIST_FEATURES}
###         do
###             # Exploitation des proprietes InstallShield "Active" (champ 2 de la sous-liste)
###             # et "ActiveForUninstall" (champ 3 de la sous-liste)
###             # On ne desinstalle que ce qui est marque comme deja installe
###             # Il y a exploitation des deux variables provenant d'InstallShield
###             # afin de determine ce qui est rellement installe (methode en fait non
###             # completement fiable) 
###             PROPERTY_INSTALL=`GetFieldSublist ${FEATURES} '|' ':' ${NAME} 2`
###             PROPERTY_UNINSTALL=`GetFieldSublist ${FEATURES} '|' ':' ${NAME} 3`
###             
### 
###             DisplayTrace "    Feature \"$NAME\" :  "
###             DisplayTrace "    PROPERTY_INSTALL     : \"${PROPERTY_INSTALL}\" "
###             DisplayTrace "    PROPERTY_UNINSTALL   : \"${PROPERTY_UNINSTALL}\" "
###             DisplayTrace ""
### 
###         
###             if [ "${PROPERTY_UNINSTALL}" = "true" ]
###             then
###                 if [ "${PROPERTY_INSTALL}" = "true" ]
###                 then
###                     LIST_FEATURES_UNINSTALL="${LIST_FEATURES_UNINSTALL}${NAME} "
###                 fi
###             fi    
###         done
### 
###         # Inversion de la liste
###         LIST_FEATURES_UNINSTALL=`ReverseString "${LIST_FEATURES_UNINSTALL}"` 
###         
###         DisplayTrace "    List of features to uninstall :"
###         DisplayTrace "    LIST_FEATURES_UNINSTALL : \"${LIST_FEATURES_UNINSTALL}\" "
###         ;;
###         
###     *)
###         echo "Unknown context \"${CONTEXT}\" " 1>&2
###         RETURN_CODE=2    
###         ;;
###     esac




    # Creation des fichiers des actions de desinstallation 

    if [ ${RETURN_CODE} -eq 0 ]
    then    
    
        if [ "${CONTEXT}" = "uninst" ]
        then
        
            # Creation des fichiers des actions de desinstallation 
         
            # Creation du fichier des actions de desinstallation : inversion
            # de l'ordre des lignes du fichier d'installation
            
            ### for NAME in ${LIST_FEATURES}
            ### do
                FILE_INSTALL=${FILE_ACTIONS_INSTALL}
                FILE_UNINSTALL=${FILE_ACTIONS_UNINSTALL}
                
                ### DisplayTrace "    Feature : \"$NAME\" "
                DisplayTrace "    FILE_INSTALL     : \"${FILE_INSTALL}\" "
                DisplayTrace "    FILE_UNINSTALL   : \"${FILE_UNINSTALL}\" "
           
                ReverseFile ${FILE_INSTALL} ${FILE_UNINSTALL}
            
            ### done

        fi
    fi    





    # Creation de la liste des fichiers d'actions pour l'installation/desinstallation

    case "${CONTEXT}" in

    "inst")    

        LIST_FILES_ACTIONS_INSTALL=""
        
        LIST_FILES_ACTIONS_INSTALL="${FILE_ACTIONS_INSTALL}"
        

        DisplayTrace "    List of action files to execute for install :"
        DisplayTrace "    LIST_FILES_ACTIONS_INSTALL : \"${LIST_FILES_ACTIONS_INSTALL}\" "

        ;;


    "uninst")    

        LIST_FILES_ACTIONS_UNINSTALL=""
        
        LIST_FILES_ACTIONS_UNINSTALL="${FILE_ACTIONS_UNINSTALL}"
        

        DisplayTrace "    List of action files to execute for uninstall :"
        DisplayTrace "    LIST_FILES_ACTIONS_UNINSTALL : \"${LIST_FILES_ACTIONS_UNINSTALL}\" "

        ;;

        
    *)
        echo "Unknown context \"${CONTEXT}\" " 1>&2
        RETURN_CODE=2    
        ;;
    esac


    # Si desinstallation, recherche d'elements communs

    ###
    ### A faire dans version future
    ###    
    ### Manque liste/fichier indiquant ce qui est deja installe   
    ### Pour faire la difference avec ce que l'onv eut desinstaller   


    return ${RETURN_CODE}
    
}


###########################################################################
# InitDefaultVariables
###########################################################################

#
# Initialisation variables avec des valeur par defaut
#
# 

InitDefaultVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitDefaultVariables"


    # Initialisation au cas ou la lecture du .INI echoue
    # Il y a ensuite suppression de fichiers temporaires avec ${RM} qui doir
    # etre alors defini
    
    
    AWK=awk
    CAT=cat
    CHGRP=chgrp
    CHMOD=chmod
    CHOWN=chown
    CP=cp
    CUT=cut
    EXPR=expr
    FIND=find
    GREP=grep
    ID=id
    KILL=kill
    LN=ln
    LS=ls
    MKDIR=mkdir
    MV=mv
    PRINTF=printf
    PS=ps
    RM=rm
    RMDIR=rmdir
    SED=sed
    SORT=sort
    TAR=tar
    ### NOUVEAU ###
    TR=tr
    ### FIN NOUVEAU ###
    WC=wc
    XARGS=xargs


    # Repertoire pour les fichiers temporaires

    DIR_TEMP="${DIR_UPDATE}/temp"
    
    return ${RETURN_CODE}
}


###########################################################################
# InitVariables
###########################################################################

#
# Initialisation variables
#
# a personnaliser
#
# Amelioration : gestion des versions differentes pour une meme plateforme

InitVariables ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> InitVariables"

    
    
    # A mettre en action dans une version future du script
    USER_MASK="022"
    
    \umask ${USER_MASK}
    

    DisplayTrace "    UMASK : ${USER_MASK}"

        
    # Le fichier de configuration porte un nom specifique a chaque plateforme 
    # afin d'eviter toute confusion
    # Le nom de la plateforme est transmis par InstallShield et est "converti"
    # afin de correspondre aux noms de plateformes HPU ("sun" au lieu de 
    # "solaris", etc.)
    PLATFORM_CONVERTED=`FormatPlatformName ${PLATFORM}`
    RETURN_CODE=$?
    

    if [ ${RETURN_CODE} -eq 0 ]
    then

        DisplayTrace ""
        DisplayTrace "    PLATFORM_CONVERTED : \"${PLATFORM_CONVERTED}\""

        PLATFORM=${PLATFORM_CONVERTED}
        
        PLATFORM_CONFIG_FILE="update_${PLATFORM}.ini"
        PREFIX_CONFIG_FILE="tmp_section_"
        
    else    
    
        echo "Unidentified platform : \"${PLATFORM}\" " 1>&2
        DisplayTrace "Unidentified platform : \"${PLATFORM}\" " 

    fi
    
    
    # Repertoire pour les fichiers temporaires

    DIR_TEMP="${DIR_UPDATE}/temp"
    
    \mkdir ${DIR_TEMP}
    # RETURN_CODE=$?
    
    
    # Liste complete des features (a inst/desinstaller ou non)
    LIST_FEATURES=`echo ${FEATURES} | \tr -s '|' '\n' | \cut -d':' -f1`


    DisplayTrace "    LIST_FEATURES     : \"${LIST_FEATURES}\" "    

    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        # Decomposition fichier de configuration selon les sections
        # qu'il contient
        PrepareINIFile ${PLATFORM_CONFIG_FILE} ${PREFIX_CONFIG_FILE}  
        RETURN_CODE=$?
    fi
    
    
    FILE_ARCHIVE="archive.tar"

     
    # C'est normalement un argument d'appel ? (ou repertoire courant ?)
    # Repertoire d'ou s'execute le script d'installation / repertoire 
    # pour extraire l'archive des fichiers a installer
    ### DIRECTORY_TEMP='/home/a_ng/HPU/ScriptsInstallation/ArborescenceTest/tmp'
    
    
    DIR_ARCHIVE="${DIR_TEMP}/Files"
    
    

    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitCommandsVariables
        RETURN_CODE=$?
    fi
     
    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitSystemFilesVariables
        RETURN_CODE=$?
    fi

        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitActionFilesVariables
        RETURN_CODE=$?
    fi

### Utilise la section "directories" inutilisee (obsolete)
###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         InitDirectoriesVariables
###         RETURN_CODE=$?
###     fi

    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitOtherSystemVariables
        RETURN_CODE=$?
    fi



    return ${RETURN_CODE}
}



###########################################################################
# CheckEnvironment
###########################################################################

#
# Verification des chemins d'acces aux commandes et fichiers
#
# 

CheckEnvironment ()

{
    RETURN_CODE=0


    echo ""
    DisplayTrace ">>> CheckEnvironment"


    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${AWK}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CAT}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CHGRP}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CHMOD}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CHOWN}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${CP}
        RETURN_CODE=$?
    fi

    
###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         CheckExecutableFile ${CUT}
###         RETURN_CODE=$?
###     fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${FIND}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${GREP}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${ID}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${KILL}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${LN}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${LS}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${MKDIR}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${MV}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${PRINTF}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${PS}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${RM}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${RMDIR}
        RETURN_CODE=$?
    fi
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${SED}
        RETURN_CODE=$?
    fi
        
        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${SORT}
        RETURN_CODE=$?
    fi
        

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${TAR}
        RETURN_CODE=$?
    fi
        
        


    
###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         ###         CheckWriteableFile ${FILE_SERVICES}
###         CheckReadableFile ${FILE_SERVICES}
###         RETURN_CODE=$?
###     fi
###     
###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         ###         CheckWriteableFile ${FILE_PROFILE}
###         CheckReadableFile ${FILE_PROFILE}
###         RETURN_CODE=$?
###     fi
###     
###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         ###         CheckWriteableFile ${FILE_STARTUP}
###         CheckReadableFile ${FILE_STARTUP}
###         RETURN_CODE=$?
###     fi
###     
###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         
###         # Le fichier de configuration inetd n'est pas le meme selon la distribution
###         # Linux ou zLinux : son nom est reconfigure ulterieurement
###         if [ "${PLATFORM}" != "linux" -a "${PLATFORM}" != "zlinux" ]
###         then
###             CheckReadableFile ${FILE_INET_SERVICES}
###             RETURN_CODE=$?
###         fi
###     fi

    return ${RETURN_CODE}

}




###########################################################################
# ConvertVersionToNumeric
###########################################################################

#
# Conversion du texte de numero de version ("NN.NN.NNN") en nombre NNNNNNN
#
# 
# Cette fonction affiche le resultat sur stdout : a utiliser avec une 
# redirection 
# Aucun affichage parasite sur stdout ne doit etre fait dans cette fonction.
# 
# 

ConvertVersionToNumeric ()
{
    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> ConvertVersionToNumeric"
    
    
    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
        
    else    
    
        VERSION_NUMBER_TEMP=$1


        VERSION_NUMBER_TEMP_MAJOR=`echo ${VERSION_NUMBER_TEMP} | cut -d'.' -f1`
        VERSION_NUMBER_TEMP_MINOR=`echo ${VERSION_NUMBER_TEMP} | cut -d'.' -f2`
        VERSION_NUMBER_TEMP_SUB=`echo ${VERSION_NUMBER_TEMP} | cut -d'.' -f3`

        VERSION_NUMBER_TEMP_MAJOR=`${EXPR} ${VERSION_NUMBER_TEMP_MAJOR} + 0`
        VERSION_NUMBER_TEMP_MINOR=`${EXPR} ${VERSION_NUMBER_TEMP_MINOR} + 0`
        VERSION_NUMBER_TEMP_SUB=`${EXPR} ${VERSION_NUMBER_TEMP_SUB} + 0`

### DEBUG ###
echo "VERSION_NUMBER_TEMP_MAJOR : \"${VERSION_NUMBER_TEMP_MAJOR}\" " 1>&2
echo "VERSION_NUMBER_TEMP_MINOR : \"${VERSION_NUMBER_TEMP_MINOR}\" " 1>&2
echo "VERSION_NUMBER_TEMP_SUB   : \"${VERSION_NUMBER_TEMP_SUB}\" " 1>&2
### FIN DEBUG ###


        VERSION_NUMERIC=0
        TEMP_CALC=`expr ${VERSION_NUMBER_TEMP_MAJOR} "*" 100000`
        VERSION_NUMERIC=`expr ${VERSION_NUMERIC} + ${TEMP_CALC}`
        TEMP_CALC=`expr ${VERSION_NUMBER_TEMP_MINOR} "*" 1000`
        VERSION_NUMERIC=`expr ${VERSION_NUMERIC} + ${TEMP_CALC}` 
        TEMP_CALC="${VERSION_NUMBER_TEMP_SUB}"
        VERSION_NUMERIC=`expr ${VERSION_NUMERIC} + ${TEMP_CALC}` 


        echo "${VERSION_NUMERIC}"
    fi
    
    RETURN_CODE=$?
}





###########################################################################
# CompareVersions
###########################################################################

# Fct abandonnee

# InitialXXXXisation variables avec des valeur par defaut
#
# 

CompareVersions ()
{
    RETURN_CODE=0
    
    VERSION_NUMBER_1=$1
    VERSION_NUMBER_2=$2


    VERSION_NUMBER_1_MAJOR=`echo ${VERSION_NUMBER_1} | cut -d'.' -f1`
    VERSION_NUMBER_1_MINOR=`echo ${VERSION_NUMBER_1} | cut -d'.' -f2`
    VERSION_NUMBER_1_SUB=`echo ${VERSION_NUMBER_1} | cut -d'.' -f3`

    VERSION_NUMBER_1_MAJOR=`${EXPR} ${VERSION_NUMBER_1_MAJOR} + 0`
    VERSION_NUMBER_1_MINOR=`${EXPR} ${VERSION_NUMBER_1_MINOR} + 0`
    VERSION_NUMBER_1_SUB=`${EXPR} ${VERSION_NUMBER_1_SUB} + 0`

    VERSION_NUMBER_2_MAJOR=`echo ${VERSION_NUMBER_2} | cut -d'.' -f1`
    VERSION_NUMBER_2_MINOR=`echo ${VERSION_NUMBER_2} | cut -d'.' -f2`
    VERSION_NUMBER_2_SUB=`echo ${VERSION_NUMBER_2} | cut -d'.' -f3`

    VERSION_NUMBER_2_MAJOR=`${EXPR} ${VERSION_NUMBER_2_MAJOR} + 0`
    VERSION_NUMBER_2_MINOR=`${EXPR} ${VERSION_NUMBER_2_MINOR} + 0`
    VERSION_NUMBER_2_SUB=`${EXPR} ${VERSION_NUMBER_2_SUB} + 0`

    if [ ${VERSION_NUMBER_1} = ${VERSION_NUMBER_2} ]
    then
    
        echo "Version identiques"
        RETURN_CODE=0
    
     
    else

        if [ ${VERSION_NUMBER_1_MAJOR} -gt ${VERSION_NUMBER_2_MAJOR} ]
        then
        
            echo "Version 1 > Version 2"
        else
        

            if [ ${VERSION_NUMBER_1_MAJOR} -gt ${VERSION_NUMBER_2_MAJOR} ]
            then
            :
            fi
        fi
    fi

    return ${RETURN_CODE}
}


###########################################################################
# CheckUpdatePlatform
###########################################################################

#

CheckUpdatePlatform ()
{

    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> CheckUpdatePlatform"
    

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
        
    else
        CURRENT_PLATFORM=`\uname -s | tr '[A-Z]' '[a-z]'`
        CURRENT_PLATFORM=`FormatPlatformName ${CURRENT_PLATFORM}`

        if [ "${UPDATE_OS}" != "${CURRENT_PLATFORM}" ]
        then
            echo "This update is for `UnformatPlatformName ${UPDATE_OS}` and the current system is ${PLATFORM}."
            RETURN_CODE=1
        fi
    fi


    return ${RETURN_CODE}
}



###########################################################################
# CheckVersion
###########################################################################

#
# Controle si la version du hotfix peut etre appliquee a la version courante
#
#
# Attention : 
# Les versions controlees sont sous forme de chaines de caracteres.
# Les comparaisons sont sous donc entre des chaines (suite demande) avec les 
# risques que cela comporte
#

CheckVersion ()
{

    RETURN_CODE=0
    
    
    DisplayTrace ""
    DisplayTrace ">>> CheckVersion"
    

    if [ $# -ne 3 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
        
    else
        VERSION_CURRENT=$1
        VERSION_HOTFIX=$2
        VERSION_UPDATE_BASE=$3
        
        DisplayTrace "    VERSION_CURRENT     : ${VERSION_CURRENT}"
        DisplayTrace "    VERSION_HOTFIX      : ${VERSION_HOTFIX}"
        DisplayTrace "    VERSION_UPDATE_BASE : ${VERSION_UPDATE_BASE}"
        
        echo ""
        
        if [[ ${VERSION_CURRENT} > ${VERSION_HOTFIX} ]]
        then
            DisplayTrace "    Version ${VERSION_CURRENT} is too high to apply the update ${UPDATE_LABEL}"
            echo "Version ${VERSION_CURRENT} is too high to apply the update ${UPDATE_LABEL}"
            RETURN_CODE=8
        else
            if [[ ${VERSION_CURRENT} = ${VERSION_HOTFIX} ]]
            then
                DisplayTrace "    Version ${VERSION_CURRENT} is already installed"
                echo "Version ${VERSION_CURRENT} is already installed"
                RETURN_CODE=8
            else
                if [[ ${VERSION_CURRENT} < ${VERSION_UPDATE_BASE} ]]
                then 
                   DisplayTrace "    Version ${VERSION_CURRENT} is too low to apply the update ${UPDATE_LABEL}"
                   echo "Version ${VERSION_CURRENT} is too low to apply the update ${UPDATE_LABEL}"
                   RETURN_CODE=8
                else
                   DisplayTrace "    Found version ${VERSION_CURRENT}, update ${UPDATE_LABEL} to version ${VERSION_HOTFIX} can be applied."
                   echo "Found version ${VERSION_CURRENT}, update ${UPDATE_LABEL} to version ${VERSION_HOTFIX} can be applied."
                   RETURN_CODE=0
                fi
           fi
       fi

    fi
    
    return ${RETURN_CODE}
}    


###########################################################################
# CheckVersionObsolete
###########################################################################

#
# Controle si la version du hotfix peut etre appliquee a la version courante
#
#  Inutilisee


CheckVersionObsolete ()
{

    RETURN_CODE=0
    
    
    # DisplayTrace ""
    # DisplayTrace ">>> CheckVersionObsolete"
    

    if [ $# -ne 3 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        echo '\07Error'
        RETURN_CODE=1
        
    else
        VERSION_CURRENT=$1
        VERSION_HOTFIX=$2
        VERSION_UPDATE_BASE=$3
        
        echo "VERSION_CURRENT : ${VERSION_CURRENT}"
        echo "VERSION_HOTFIX : ${VERSION_HOTFIX}"
        
        if [ ${VERSION_CURRENT} -gt ${VERSION_HOTFIX} ]
        then
            echo "Version ${VERSION_CURRENT} is too high to apply the update to ${UPDATE_LABEL}"
            RETURN_CODE=8
        else
            if [ ${VERSION_CURRENT} -eq ${VERSION_HOTFIX} ]
            then
                echo "Version ${VERSION_CURRENT} is already installed"
                RETURN_CODE=8
            else
                if [ ${VERSION_CURRENT} -lt ${VERSION_UPDATE_BASE} ]
                then 
                   echo "Version ${VERSION_CURRENT} is too low to apply the update ${UPDATE_LABEL}"
                   RETURN_CODE=8
                else
                   echo "Found Version ${VERSION_CURRENT}, update to ${UPDATE_LABEL} can be applied."
                   RETURN_CODE=0
                fi
           fi
       fi

    fi
    
    return ${RETURN_CODE}
}    


##########################################################################
# ExtractArchiveFile
###########################################################################

#
# Extraction des fichiers d'archive contenant les fichiers a installer
# si installation demandee
# 
# La trace de l'extraction de l'archive est redirigee dans le fichier de trace.
# 
# 
# Ajout du controlez d'existence du fichier d'archive
# 

ExtractArchiveFile ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> ExtractArchiveFile"
    
    if [ "${CONTEXT}" = "inst" ]
    then
        DIR_CURRENT=`\pwd`
    
        ${MKDIR} ${DIR_ARCHIVE}
        RETURN_CODE=$?

        if [ ${RETURN_CODE} -ne 0 ]
        then
            echo "Error while creating directory \"${DIR_ARCHIVE}\" " 1>&2
        else      
        
            # Extraction des fichiers de l'archive

            \cd ${DIR_ARCHIVE}

            FILE=${FILE_ARCHIVE}

            DisplayTrace ""
            DisplayTrace "    FILE : \"${FILE}\" "

            if [ ${RETURN_CODE} -eq 0 ]
            then
                   CheckReadableFile ${DIR_CURRENT}/${FILE}
                   RETURN_CODE=$?
            fi
            
            
            if [ ${RETURN_CODE} -eq 0 ]
            then
            
                if [ "${PLATFORM}" = "hp" ]
                then
                    # Redirection car sous HP-UX la liste des fichiers va dans 
                    # stderr
                    ${TAR} xvf "${DIR_CURRENT}/${FILE}" >> ${FILE_TRACE} 2>&1
                else
                    ${TAR} xvf "${DIR_CURRENT}/${FILE}" >> ${FILE_TRACE}
                fi

                if [ $? -ne 0 ]
                then
                    echo "Error while extracting archive file \"${FILE}" 1>&2
                    RETURN_CODE=1
                fi
            fi

            if [ ${RETURN_CODE} -eq 0 ]
            then
                # Suppression de l'archive au fur et a mesure
                ### DEBUG ###
                ### ${RM} "${DIR_CURRENT}/${FILE}"
                :
                ### FIN DEBUG ###
            fi
            
            \cd ${DIR_CURRENT}
        
        fi    
    
    fi
    
    return ${RETURN_CODE}
}





##########################################################################
# ExtractArchive
###########################################################################

# 
# Extraction du fichier d'archive contenant les fichiers a installer
# si installation demandee
# 
# 
# 

ExtractArchive ()
{
    RETURN_CODE=0
    
    DisplayTrace ""
    DisplayTrace ">>> ExtractArchive"
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        InitArchiveFileVariable
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        ExtractArchiveFile
        RETURN_CODE=$?
    fi
    
    
    return ${RETURN_CODE}
}



###########################################################################
# UpdateDirectoryINST
###########################################################################

# 
# InstCXXXallation repertoires (installation)
#
# Actuellement : pas de filtres (metacaracteres) supportes 
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_directory"
#
#
# Le contenu actuel du repertoire n'est pas supprime.
# Pas d'option "include_subdir" : on ne gere pas les sous-repertoires
#
# A faire : controle validite de SUBDIR_INCLUSION
# A faire : Proprietaire/permissions ???!!!
#
#

UpdateDirectoryINST ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> UpdateDirectoryINST"


    UPDATE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "update_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 
    OWNER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "owner"`


    DisplayTrace "    LOCATION_PATH    : ${LOCATION_PATH}"
    DisplayTrace "    UPDATE_DIR       : ${UPDATE_DIR}"
    DisplayTrace "    SOURCE_DIR       : ${SOURCE_DIR}"
    DisplayTrace "    OWNER            : ${OWNER}"

    # Si le repertoire n'est pas dans l'archive extraite, on ne le traite pas
    # (Pas d'erreur pour plus de souplesse)
    
    if [ -d "${DIR_ARCHIVE}/${SOURCE_DIR}" ]
    then

        if [ ! -d ${LOCATION_PATH}/${UPDATE_DIR} ]
        then
            DisplayTrace "    Creating directory\"${LOCATION_PATH}/${UPDATE_DIR}\" "
            ${MKDIR} "${LOCATION_PATH}/${UPDATE_DIR}"
        fi


        # Copie des fichiers seuls sans les repertoires

        # Pas de guillemets si metacaractere utilise
        # ${CP} -p ${DIR_ARCHIVE}/${SOURCE_DIR}/* "${LOCATION_PATH}/${INSTALL_DIR}"

        ${LS} -l "${DIR_ARCHIVE}/${SOURCE_DIR}" | ${GREP} -v '^d' | ${SED} -e '1d' | ${AWK} '{print $NF}' | \
        while read FILE
        do

            DisplayTrace "" 
            DisplayTrace "    FILE : \"${FILE}\" "

            if [ -f "${LOCATION_PATH}/${UPDATE_DIR}/${FILE}_${UPDATE_LABEL}" ]
            then
                DisplayTrace "    Error : file \"${LOCATION_PATH}/${UPDATE_DIR}/${FILE}\" already updated with ${UPDATE_LABEL}"
                echo "    Error : file \"${LOCATION_PATH}/${UPDATE_DIR}/${FILE}\" already updated with ${UPDATE_LABEL}" 1>&2

            else
                if [ -f "${LOCATION_PATH}/${UPDATE_DIR}/${FILE}" ]
                then

                    DisplayTrace "    Existing file"

                    # Renommage du fichier actuel 

                    DisplayTrace "    Renaming \"${LOCATION_PATH}/${UPDATE_DIR}/${FILE}\" to \"${LOCATION_PATH}/${UPDATE_DIR}/${FILE}_${UPDATE_LABEL}\""
                    DisplayTrace ""

                    ${MV} "${LOCATION_PATH}/${UPDATE_DIR}/${FILE}" "${LOCATION_PATH}/${UPDATE_DIR}/${FILE}_${UPDATE_LABEL}"

                    # Copie du fichier

                    DisplayTrace "    Copying \"${DIR_ARCHIVE}/${SOURCE_DIR}/${FILE}\" to \"${LOCATION_PATH}/${UPDATE_DIR}\""
                    DisplayTrace ""

                    ${CP} -p "${DIR_ARCHIVE}/${SOURCE_DIR}/${FILE}" "${LOCATION_PATH}/${UPDATE_DIR}"

                else
                    # Fichier absent 

                    # Copie du fichier

                    DisplayTrace "    New file"
                    DisplayTrace "    Copying \"${DIR_ARCHIVE}/${SOURCE_DIR}/${FILE}\" to \"${LOCATION_PATH}/${UPDATE_DIR}\""
                    DisplayTrace ""

                    ${CP} -p "${DIR_ARCHIVE}/${SOURCE_DIR}/${FILE}" "${LOCATION_PATH}/${UPDATE_DIR}"

                    # On cree un fichier vide correspondant a son ancienne version afin d'avoir une trace de son 
                    # installation lors de la desinstallation

                    DisplayTrace "    Creating \"${LOCATION_PATH}/${UPDATE_DIR}/${FILE}_${UPDATE_LABEL}\" "
                    DisplayTrace ""

                    > ${LOCATION_PATH}/${UPDATE_DIR}/${FILE}_${UPDATE_LABEL}

                    # La date du fichier est celle de la creation et non celle du fichier "original"

                fi
            fi
        done   

    else
        DisplayTrace "    No file to update"
    fi
        

###    if [ "${OWNER}" != "" ]
###    then
###        ${CHOWN} -R ${OWNER} ${LOCATION_PATH}/${UPDATE_DIR}
###    fi            


    return ${RETURN_CODE}
}


###########################################################################
# UpdateDirectoryUNINST
###########################################################################

# 
# Suppression repertoires (desinstallation)
#
# Il y a suppression du repertoire et de sons contenu de maniere recursive
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_directory"
#
#
# Pas d'option "include_subdir" : on ne gere pas les sous-repertoires
#
#

UpdateDirectoryUNINST ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> UpdateDirectoryUNINST"
    


    UPDATE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "update_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 
    OWNER=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "owner"`



    DisplayTrace "    LOCATION_PATH    : ${LOCATION_PATH}"
    DisplayTrace "    UPDATE_DIR       : ${UPDATE_DIR}"
    DisplayTrace "    SOURCE_DIR       : ${SOURCE_DIR}"
    DisplayTrace "    OWNER            : ${OWNER}"



    DIR_CURRENT=`pwd`

    if [ -d "${LOCATION_PATH}/${UPDATE_DIR}" ]
    then

        \cd ${LOCATION_PATH}/${UPDATE_DIR}

        # Tester le "cd" !!!
          
          
          
        # Risque overflow si trop de fichiers  
        FILE_LIST=`${LS} -1 *_${UPDATE_LABEL} 2>/dev/null`
        if [ -z "${FILE_LIST}" ]
        then
            DisplayTrace "" 
            DisplayTrace "    No file to restore !" 
        else
            for FILE in ${FILE_LIST}
            do

                DisplayTrace "" 
                DisplayTrace "    FILE : \"${FILE}\" "

                if [ -s "${FILE}" ]
                then

                    # Fichier non vide : a restaurer

                    DisplayTrace "    Previous existing file to restore"

                    # Renommage du fichier actuel 


                    # Obtention du nom original a partir du nom de la copie 
                    
                    # Sous Linux le "^" en tete de '^\(.*\)'"_${UPDATE_LABEL}"provoque un 
                    # avertissement :
                    #
                    # expr: warning: unportable BRE: `^\(.*\)_': using `^' as the first character
                    # of the basic regular expression is not portable; it is being ignored   
                    #
                    # Le "^" serait inutile.
                    # Comme cela fonctionne sans probleme sur les autres plateformes,
                    # on cree un cas particulier pour Linux.
                    # A priori on devrait pouvoir retirer le "^" sur toutes les
                    # plateformes.
                    
                    if [ "${PLATFORM}" = "linux" ]
                    then
                        ORIGINAL_NAME=`${EXPR} "${FILE}" ":" '\(.*\)'"_${UPDATE_LABEL}" `
                    else
                        ORIGINAL_NAME=`${EXPR} "${FILE}" ":" '^\(.*\)'"_${UPDATE_LABEL}" `
                    fi    

                    DisplayTrace "    ORIGINAL_NAME : \"${ORIGINAL_NAME}\" "

                    DisplayTrace "    Restoring from \"${LOCATION_PATH}/${UPDATE_DIR}/${FILE}\" "
                    DisplayTrace ""

                    ${MV} "${FILE}" "${ORIGINAL_NAME}"

                else
                    # Fichier vide : pas de restauration a effectuer (le fichier n'a pas d'ancienne version)

                    # Obtention du nom original a partir du nom de la copie 
                    
                    # Sous Linux le "^" en tete de '^\(.*\)'"_${UPDATE_LABEL}"provoque un 
                    # avertissement :
                    #
                    # expr: warning: unportable BRE: `^\(.*\)_': using `^' as the first character
                    # of the basic regular expression is not portable; it is being ignored   
                    #
                    # Le "^" serait inutile.
                    # Comme cela fonctionne sans probleme sur les autres plateformes,
                    # on cree un cas particulier pour Linux.
                    # A priori on devrait pouvoir retirer le "^" sur toutes les
                    # plateformes.
                    
                    if [ "${PLATFORM}" = "linux" ]
                    then
                        ORIGINAL_NAME=`${EXPR} "${FILE}" ":" '\(.*\)'"_${UPDATE_LABEL}" `
                    else
                        ORIGINAL_NAME=`${EXPR} "${FILE}" ":" '^\(.*\)'"_${UPDATE_LABEL}" `
                    fi    
                    

                    DisplayTrace "    ORIGINAL_NAME : \"${ORIGINAL_NAME}\" "


                    # Suppression fichier actuel

                    DisplayTrace ""  
                    DisplayTrace "    Removing \"${LOCATION_PATH}/${UPDATE_DIR}/${FILE}\" "
                    DisplayTrace ""

                    ${RM} "${ORIGINAL_NAME}"

                    # Suppression fichier a restorer (qui est detaille 0)

                    DisplayTrace "    No previous existing file to restore "
                    DisplayTrace "    Removing \"${LOCATION_PATH}/${UPDATE_DIR}/${FILE}\" "
                    DisplayTrace ""

                    ${RM} "${FILE}"

                fi

            done  
        fi 
        
    else
        DisplayTrace ""
        DisplayTrace "   Directory \"${LOCATION_PATH}/${UPDATE_DIR}\" not found."
        DisplayTrace "   No file to restore."
        
    fi    

        \cd ${DIR_CURRENT}  

    ######### ${RM} -Rf "${LOCATION_PATH}/${INSTALL_DIR}"
    

    return ${RETURN_CODE}
}


###########################################################################
# UpdateDirectory
###########################################################################

# 
# Installation repertoires
#
# Actuellement : pas de filtres (metacaracteres) supportes 
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_directory"
#
#
# A faire : controle validite de SUBDIR_INCLUSION
#
#

UpdateDirectory ()

{
    RETURN_CODE=0



    DisplayTrace ""
    DisplayTrace ">>> UpdateDirectory"

    
    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 
        
        # Controle de compatibilite de la section avec la commande
        if [ "${SECTION_TYPE}" != "update_directory" ]
        then
            echo "Incompatible section type \"${SECTION_TYPE}\" with action" 1>&2
            RETURN_CODE=1
        else

            case "${CONTEXT}" in 
            "inst")
                UpdateDirectoryINST $@
                RETURN_CODE=$?
                ;;  

            "uninst")
                UpdateDirectoryUNINST $@
                RETURN_CODE=$?
                ;;  

            *)
                echo "Unknown context \"${CONTEXT}\" " 1>&2
                RETURN_CODE=2
                ;;  
            esac
        fi    
    
    fi

    return ${RETURN_CODE}
}





###########################################################################
# UpdateFilesINST
###########################################################################

# 
# Mise-a-jour fichiers (installation)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_files"
#
# 
# XXXXXXXXXXXXSi le fichier existe deja dans le repertoire destination, il n'est pas copie
#

UpdateFilesINST ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> UpdateFilesINST"

    
    UPDATE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "update_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 


    DisplayTrace "    UPDATE_DIR    : ${UPDATE_DIR}"
    DisplayTrace "    SOURCE_DIR    : ${SOURCE_DIR}"

    
    # Pas d'utilisation de la fonction CheckExistingDirectory () car un 
    # message d'erreur est affiche si inexistant
    if [ ! -d "${LOCATION_PATH}/${UPDATE_DIR}" ]
    then

        DisplayTrace "    Creation of directory \"${LOCATION_PATH}/${UPDATE_DIR}\" " 

        
        ${MKDIR} "${LOCATION_PATH}/${UPDATE_DIR}"
    fi
    
    ${CAT} "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" | ${SED} -e '1d' | while read LINE 
    do
        if [ "${LINE}" != "" ]
        then

            # DisplayTrace "    LINE : \"${LINE}\"" 


            IDENTIFIER=`echo ${LINE} | cut -d'=' -f1`
            VALUE=`echo ${LINE} | cut -d'=' -f2`

            if [ "${IDENTIFIER}" != "file" ]
            then

                # DisplayTrace "    Line discarded"

                continue
            else

                DisplayTrace "    LINE : \"${LINE}\"" 
                
            fi

            FILE=`echo ${VALUE} | cut -d':' -f1`
            PERM=`echo ${VALUE} | cut -d':' -f2`
            OWNER=`echo ${VALUE} | cut -d':' -f3`
            GROUP=`echo ${VALUE} | cut -d':' -f4`


            DisplayTrace ""
            DisplayTrace "    FILE  : \"${FILE}\"" 
            DisplayTrace "    PERM  : \"${PERM}\"" 
            DisplayTrace "    OWNER : \"${OWNER}\"" 
            DisplayTrace "    GROUP : \"${GROUP}\"" 


            # Si la ligne ne comporte aucun separateur de champ, 
            # le "cut" renvoie le nom de fichier dans tous les champs
            if [ "${FILE}" = "${GROUP}" ]
            then
                echo "Incorrect value format in line \"${LINE}\" " 1>&2
                RETURN_CODE=1
                break
            fi



###             ${CP} -p "${DIR_ARCHIVE}/${SOURCE_DIR}/${FILE}" "${LOCATION_PATH}/${INSTALL_DIR}"
###             if [ $? -ne 0 ]
###             then
###                 echo "Error while copying file \"${FILE}\" " 1>&2
###                 return 1
###             fi   

            # Si le fichier existe deja dans la repertoire destination, ne
            # rieCXXXXn faire (il peut avoir ete deja installe par une action 
            # precedente)
            if [ -f "${LOCATION_PATH}/${UPDATE_DIR}/${FILE}" ]
            then
                DisplayTrace "    File \"${FILE}\" already found at ${LOCATION_PATH}/${UPDATE_DIR}."
                ### DisplayTrace "    Not copied."
                
                # Deplacement du fichier dans le repertoire de backup

                ${MV} "${LOCATION_PATH}/${UPDATE_DIR}/${FILE}" "${LOCATION_PATH}/${BACKUP_DIR}"
                
                # Copie du fichier
                          
                ${CP} -p "${DIR_ARCHIVE}/${SOURCE_DIR}/${FILE}" "${LOCATION_PATH}/${UPDATE_DIR}"
                
            else
                # Fichier absent 
                echo "Error while copying file \"${FILE}\" " 1>&2
                return 1
            
            fi
            
            

            if [ "${PERM}" != "" ]
            then
                SetPermissionsFile ${PERM} ${LOCATION_PATH}/${INSTALL_DIR}/${FILE}
            fi

            if [ "${OWNER}" != "" ]
            then
                SetOwnerFile ${OWNER} ${LOCATION_PATH}/${INSTALL_DIR}/${FILE}
            fi

            if [ "${GROUP}" != "" ]
            then
                SetGroupFile ${GROUP} ${LOCATION_PATH}/${INSTALL_DIR}/${FILE}
            fi

        fi
    done


    return ${RETURN_CODE}
}


###########################################################################
# UpdateFilesUNINST
###########################################################################

#
# Desinstallation fichiers (desinstallation)
#
# Test existence fichier destination a faire !
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_files"
#
# Pas de suppression du repertoire ?
#

UpdateFilesUNINST ()
{
    RETURN_CODE=0

    
    DisplayTrace ""
    DisplayTrace ">>> UpdateFilesUNINST"
    
    
    UPDATE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "update_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 


    DisplayTrace "    UPDATE_DIR   : ${UPDATE_DIR}"
    DisplayTrace "    SOURCE_DIR   : ${SOURCE_DIR}"

    
    ${CAT} "${DIR_TEMP}/${PREFIX_CONFIG_FILE}$1" | ${SED} -e '1d' | while read LINE 
    do
        if [ "${LINE}" != "" ]
        then

            # DisplayTrace "    LINE : \"${LINE}\"" 


            IDENTIFIER=`echo ${LINE} | ${CUT} -d'=' -f1`
            VALUE=`echo ${LINE} | ${CUT} -d'=' -f2`

            if [ "${IDENTIFIER}" != "file" ]
            then

                # DisplayTrace "    Line discarded"

                continue
            else

                DisplayTrace "    LINE : \"${LINE}\"" 

            fi

            FILE=`echo ${VALUE} | ${CUT} -d':' -f1`
            PERM=`echo ${VALUE} | ${CUT} -d':' -f2`
            OWNER=`echo ${VALUE} | ${CUT} -d':' -f3`
            GROUP=`echo ${VALUE} | ${CUT} -d':' -f4`


            DisplayTrace ""
            DisplayTrace "    FILE  : \"${FILE}\"" 
            DisplayTrace "    PERM  : \"${PERM}\"" 
            DisplayTrace "    OWNER : \"${OWNER}\"" 
            DisplayTrace "    GROUP : \"${GROUP}\"" 


            # Si la ligne ne comporte aucun separateur de champ, 
            # le "cut" renvoie le nom de fichier dans tous les champs
            if [ "${FILE}" = "${GROUP}" ]
            then
                echo "Incorrect value format in line \"${LINE}\" " 1>&2
                RETURN_CODE=1
                break
            fi


            ${RM} -f "${LOCATION_PATH}/${INSTALL_DIR}/${FILE}"
            if [ $? -ne 0 ]
            then
                echo "Error while deleting file \"${FILE}\" " 1>&2
                # return 1
            fi   
            

        fi
    done


    # Suppression du repertoire si vide
    if CheckEmptyDir ${LOCATION_PATH}/${INSTALL_DIR}
    then

        DisplayTrace "    Deletion of directory \"${LOCATION_PATH}/${INSTALL_DIR}\" " 

    
        ${RMDIR} "${LOCATION_PATH}/${INSTALL_DIR}"
    else
        # Variable positionnee par CheckEmptyDir ()
        RETURN_CODE=0    
    fi        


    return ${RETURN_CODE}
}


###########################################################################
# UpdateFiles
###########################################################################

#
# Installation fichiers
#
# Test existence fichier destination a faire !
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_files"
#
# 
#

UpdateFiles ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> UpdateFiles"
    
    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 
        
        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"
        
        # Controle de compatibilite de la section avec la commande
        if [ "${SECTION_TYPE}" != "update_files" ]
        then
            echo "Incompatible section type \"${SECTION_TYPE}\" with action" 1>&2
            RETURN_CODE=1
        else

            case "${CONTEXT}" in 
            "inst")
                UpdateFilesINST $@
                RETURN_CODE=$?
                ;;  

            "uninst")
                UpdateFilesUNINST $@
                RETURN_CODE=$?
                ;;  

            *)
                echo "Unknown context \"${CONTEXT}\" " 1>&2
                RETURN_CODE=2
                ;;  
            esac
        fi    
    
    fi

    return ${RETURN_CODE}
}


###########################################################################
# UpdatePlugInINST
###########################################################################

# 
# Mise-a-jour fichiers du plug-in (installation)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_plugin"
#
# 
#
# L'arborescence des classes java est entierement remplacee : il y a suppression au prealable.
# Pas de mise a jour partielle.
#
# >>> Parametres de l'action inutilises pour l'instant
#
# Par rapport a MAJ 2.1.x : pas de classes differentes selon version DB2 V7/V8
# Messages d'erreur a ameliorer
#
#
#

UpdatePlugInINST ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> UpdatePlugInINST"

    
    UPDATE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "update_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 


    DisplayTrace "    UPDATE_DIR    : ${UPDATE_DIR}"
    DisplayTrace "    SOURCE_DIR    : ${SOURCE_DIR}"

    
    ZIP=${LOCATION_PATH}/install/zip

    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${ZIP}
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        DB2_VERSION=`${GREP} '^db2version=' ${LOCATION_PATH}/cfg/db2hpu.cfg | ${AWK} -F'=' '{print $2}'`
        if [ "${DB2_VERSION}" = "" ] 
        then
            DisplayTrace "Error: No plugin needed" 
            echo "Error: No plugin needed" 1>&2
            RETURN_CODE=8
        fi
     fi
     
     
    if [ ${RETURN_CODE} -eq 0 ]
    then
    
        # Determination reprtoire DB2 du plugin 
        
        SQL_DIR=`${GREP} '^insthomes=' ${LOCATION_PATH}/cfg/db2hpu.cfg | ${AWK} -F':' '{print $2}' | ${AWK} -F',' '{print $1}'`/sqllib
        case ${DB2_VERSION} in
        "V7")
            DB2_DIR=${SQL_DIR}/cc
            ;;
            
        "V8")
            DB2_DIR=${SQL_DIR}/tools
            ;;
            
        *)
            DisplayTrace "Error: Unknown DB2 plugin version ${DB2_VERSION} in configuration file"
            echo "Error: Unknown DB2 plugin version ${DB2_VERSION} in configuration file" 1>&2
            RETURN_CODE=8
            ;;
        esac
    fi   
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        if [ ! -d ${DB2_DIR} ]
        then
            DisplayTrace "Error: Plugin install directory ${DB2_DIR} not found" 
            echo "Error: Plugin install directory ${DB2_DIR} not found" 1>&2
            RETURN_CODE=8
        fi
    fi   
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        if [ ! -f ${DB2_DIR}/db2plug.zip ]
        then
            DisplayTrace "Error: Plugin archive file not found"
            echo "Error: Plugin archive file not found" 1>&2
            RETURN_CODE=8
        fi
    fi   
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then

        # Utile ???
        if [ -d ${LOCATION_PATH}/java/classes_${UPDATE_LABEL} ]
        then
            DisplayTrace "Error: Plugin already updated with update ${UPDATE_LABEL}"
            echo "Error: Plugin already updated with update ${UPDATE_LABEL}"
            RETURN_CODE=8
        fi
    fi   

    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        if [ ! -x ${DIR_ARCHIVE}/java/classes/com ]
        then
            # Pas de repertoire com dans l'archive extraite 
            # Pas de generation d'erreur
            DisplayTrace ""
            DisplayTrace "Warning: no new plugin classes to be applied"
            
        else   

            DIR_CURRENT=`pwd`

            \cd ${LOCATION_PATH}/java

            # Sauvegarde de la version actuelle des classes

            ${CP} -pr classes classes_${UPDATE_LABEL}

            ${CP} -pr ${DIR_ARCHIVE}/java/classes/com classes

            \cd classes

            # Suppression classes actuelles dans archive du plugin
            DisplayTrace "    Removing plug-in classes"

            ${ZIP} -rq ${DB2_DIR}/db2plug.zip -d com/ibm/inz*

            # ${RM} -fr com

            # \cd ..
            # ${CP} -pr ${DIR_ARCHIVE}/java/classes/com classes
            # \cd classes

            # Ajout nouvelles classes dans archive du plugin
            DisplayTrace "    Adding new plug-in classes"

            ${ZIP} -rq0 ${DB2_DIR}/db2plug.zip . -i *.class

            \cd ${DIR_CURRENT}
        fi
        

    fi   
    
    
 

    return ${RETURN_CODE}
}


###########################################################################
# UpdatePlugInUNINST
###########################################################################

# 
# Mise-a-jour fichiers u plug-in (desinstallation)
#
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_plugin"
#
# 
#
# L'arborescence des classes java est entierement remplacee : il y a suppression au prealable.
# Pas de mise a jour partielle.
#
# >>> Parametres de l'action inutilises pour l'instant
#
# Par rapport a MAJ 2.1.x : pas de classes differentes selon version DB2 V7/V8
# Messages d'erreur a ameliorer
#
#
#

UpdatePlugInUNINST ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> UpdatePlugInUNINST"

    
    UPDATE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "update_subdir"` 
    SOURCE_DIR=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "source_subdir"` 


    DisplayTrace "    UPDATE_DIR    : ${UPDATE_DIR}"
    DisplayTrace "    SOURCE_DIR    : ${SOURCE_DIR}"

    

    ZIP=${LOCATION_PATH}/install/zip
    if [ ${RETURN_CODE} -eq 0 ]
    then
        CheckExecutableFile ${ZIP}
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then
        DB2_VERSION=`${GREP} '^db2version=' ${LOCATION_PATH}/cfg/db2hpu.cfg | ${AWK} -F'=' '{print $2}'`
        if [ "${DB2_VERSION}" = "" ] 
        then
            DisplayTrace "Error: No plugin needed" 
            echo "Error: No plugin needed" 1>&2
            RETURN_CODE=8
        fi
     fi
     
     
    if [ ${RETURN_CODE} -eq 0 ]
    then
    
        # Determination repertoire DB2 du plugin 
        
        SQL_DIR=`${GREP} '^insthomes=' ${LOCATION_PATH}/cfg/db2hpu.cfg | ${AWK} -F':' '{print $2}' | ${AWK} -F',' '{print $1}'`/sqllib
        case ${DB2_VERSION} in
        "V7")
            DB2_DIR=${SQL_DIR}/cc
            ;;
            
        "V8")
            DB2_DIR=${SQL_DIR}/tools
            ;;
            
        *)
            DisplayTrace "Error: Unknown DB2 plugin version ${DB2_VERSION} in configuration file"
            echo "Error: Unknown DB2 plugin version ${DB2_VERSION} in configuration file" 1>&2
            RETURN_CODE=8
            ;;
        esac
    fi   
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        
        if [ ! -d ${DB2_DIR} ]
        then
            DisplayTrace "Error: Plugin install directory ${DB2_DIR} not found" 
            echo "Error: Plugin install directory ${DB2_DIR} not found" 1>&2
            RETURN_CODE=8
        fi
    fi   
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        
        if [ ! -f ${DB2_DIR}/db2plug.zip ]
        then
            DisplayTrace "Error: Plugin archive file not found"
            echo "Error: Plugin archive file not found" 1>&2
            RETURN_CODE=8
        fi
    fi   
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then

        # Si une restauration a deja eu lieu, le repertoire de sauvegarde des
        # classes plugin a ete supprime
        if [ ! -d ${LOCATION_PATH}/java/classes_${UPDATE_LABEL} ]
        then
            DisplayTrace "    Plugin already restored from update ${UPDATE_LABEL}"
            echo "    Plugin already restored from update ${UPDATE_LABEL}" 1>&2
            
            # Pas d'erreur dans ce cas 
        else

            DIR_CURRENT=`\pwd`

            \cd ${LOCATION_PATH}/java
            if [ $? -ne 0 ]
            then
                DisplayTrace "    Error while changing to directory \"${LOCATION_PATH}/java\""
                echo "    Error while changing to directory \"${LOCATION_PATH}/java/\"" 1>&2
                RETURN_CODE=1
            fi
            
            
            if [ ${RETURN_CODE} -eq 0 ]
            then

                ${RM} -fR classes
                ${CP} -pr classes_${UPDATE_LABEL} classes
                ${RM} -fR classes_${UPDATE_LABEL}
                
                \cd classes

                if [ $? -ne 0 ]
                then
                    DisplayTrace "    Error while changing to classes directory"
                    echo "    Error while changing to classes directory" 1>&2
                    RETURN_CODE=1
                fi
            fi
                
                           
            if [ ${RETURN_CODE} -eq 0 ]
            then
                                
                # Suppression classes actuelles dans archive du plugin
                DisplayTrace "    Removing plug-in classes"

                ${ZIP} -rq ${DB2_DIR}/db2plug.zip -d com/ibm/inz*


                # Restauration classes dans archive du plugin
                DisplayTrace "    Restoring plug-in classes"

                ${ZIP} -rq0 ${DB2_DIR}/db2plug.zip . -i *.class
            fi

            \cd ${DIR_CURRENT}
       fi

    fi
     

    return ${RETURN_CODE}
}


###########################################################################
# UpdatePlugIn
###########################################################################

#
# Installation fichiers
#
# Test existence fichier destination a faire !
#
# Argument 1 : nom de section (i.e. categorie de fichiers)
#
# Type de section concernee : "update_plugin"
#
# 
#

UpdatePlugIn ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> UpdatePlugIn"
    
    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
    
        SECTION_TYPE=`GetINIValue "${DIR_TEMP}/${PREFIX_CONFIG_FILE}${1}" "section_type"` 
        
        DisplayTrace "    Section       : \"$1\" "
        DisplayTrace "    SECTION_TYPE  : ${SECTION_TYPE}"
        
        # Controle de compatibilite de la section avec la commande
        if [ "${SECTION_TYPE}" != "update_plugin" ]
        then
            echo "Incompatible section type \"${SECTION_TYPE}\" with action" 1>&2
            RETURN_CODE=1
        else

            case "${CONTEXT}" in 
            "inst")
                UpdatePlugInINST $@
                RETURN_CODE=$?
                ;;  

            "uninst")
                UpdatePlugInUNINST $@
                RETURN_CODE=$?
                ;;  

            *)
                echo "Unknown context \"${CONTEXT}\" " 1>&2
                RETURN_CODE=2
                ;;  
            esac
        fi    
    
    fi

    return ${RETURN_CODE}
}


###########################################################################
# DeleteTemporaryFiles
###########################################################################

#
# Suppression des fichiers temporaires 
#
# 
# DIR_TEMP doit etre defini
# 
# 
# RETURN_CODE non affecte sinon modification globale de sa valeur (cette 
# fonction est appelee systematiquement meme si RETURN_CODE est different de 0)
# 

DeleteTemporaryFiles ()
{

    # RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> DeleteTemporaryFiles"
    # DisplayTrace "FILE_ACTIONS_UNINSTALL : \"${FILE_ACTIONS_UNINSTALL}\" "


    # Fichiers du repertoire temp
    ${RM} -R ${DIR_TEMP} >/dev/null 2>&1



    # return ${RETURN_CODE}
}




###########################################################################
# ExecuteActions
###########################################################################

#
# Execution des actions selon le mode d'utilisation 
# (installation ou desinstallation)
#
#
# 
# 
# 

ExecuteActions ()
{
    RETURN_CODE=0

    DisplayTrace ""
    DisplayTrace ">>> ExecuteActions"
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        case "${CONTEXT}" in
        "inst")
            ExecuteActionsInstallation
            ;;

        "uninst")
            ExecuteActionsUninstallation
            ;;

        *)
            echo "Unknown context \"${CONTEXT}\" " 1>&2
            RETURN_CODE=1
            ;;

        esac
    fi    
    
    return ${RETURN_CODE}
}


###########################################################################
# ExecuteActionsInstallation
###########################################################################

#
# Execution des directives pour une installation
#
# 
# 
# - Execution des actions du fichier prologue d'installation
# - Execution des actions des fichiers des "features"
# - Execution des actions du fichier epilogue d'installation
# 

ExecuteActionsInstallation ()
{
    RETURN_CODE=0

    echo ""
    DisplayTrace ">>> ExecuteActionsInstallation"


    if [ ${RETURN_CODE} -eq 0 ]
    then
        echo "    Executing install prolog file"
        ExecuteActionsFile "${FILE_ACTIONS_PROLOG_INSTALL}"
        RETURN_CODE=$?
    fi   

    if [ ${RETURN_CODE} -eq 0 ]
    then
        echo "    Executing install file(s)"
        for FILE_ACTIONS in ${LIST_FILES_ACTIONS_INSTALL}
        do
            ExecuteActionsFile "${FILE_ACTIONS}"
            RETURN_CODE=$?
            
            if [ ${RETURN_CODE} -ne 0 ]
            then
                DisplayTrace "One action in \"${FILE_ACTIONS}\" returned an error"
                DisplayTrace "Execution of install file(s) aborted"
                
                echo "Execution of install file(s) aborted" 1>&2
                
                break
            fi
        done    
    fi   
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        echo "    Executing install epilog file"
        ExecuteActionsFile "${FILE_ACTIONS_EPILOG_INSTALL}"
        RETURN_CODE=$?
    fi   

    return ${RETURN_CODE}
}


###########################################################################
# ExecuteActionsUninstallation
###########################################################################

#
# Execution des directives pour une desinstallation
#
# 
# 
# - Execution des actions du fichier prologue de desinstallation
# - Execution des actions des fichiers des "features"
# - Execution des actions du fichier epilogue de desinstallation
# 
# 

ExecuteActionsUninstallation ()
{

    RETURN_CODE=0



    DisplayTrace ""
    DisplayTrace ">>> ExecuteActionsUninstallation"



    if [ ${RETURN_CODE} -eq 0 ]
    then
        echo "    Executing uninstall prolog file"
        ExecuteActionsFile "${FILE_ACTIONS_PROLOG_UNINSTALL}"
        RETURN_CODE=$?
    fi   

        
    if [ ${RETURN_CODE} -eq 0 ]
    then
        echo "    Executing uninstall file(s)"
        for FILE_ACTIONS in ${LIST_FILES_ACTIONS_UNINSTALL}
        do
     
            ExecuteActionsFile "${FILE_ACTIONS}"
            RETURN_CODE=$?
            
            if [ ${RETURN_CODE} -ne 0 ]
            then
                DisplayTrace "One action in \"${FILE_ACTIONS}\" returned an error"
                DisplayTrace "Execution of install file(s) aborted"
                
                echo "One action in \"${FILE_ACTIONS}\" returned an error" 1>&2
                echo "Execution of install file(s) aborted" 1>&2
                
                break
            fi
        done    
    fi   


    if [ ${RETURN_CODE} -eq 0 ]
    then
        echo "    Executing uninstall epilog file"
        ExecuteActionsFile "${FILE_ACTIONS_EPILOG_UNINSTALL}"
        RETURN_CODE=$?
    fi   


    return ${RETURN_CODE}
}




###########################################################################
# ExecuteActionsFile
###########################################################################

#
# Execution des directives contenues dans un fichier d'actions
#
# 
# Argument 1 : fichier des actions a executer
# 
# 
# Attention : les noms des commandes ne doivent pas debuter par une meme
# portion de texte sinon une commande risque de ne jamais etre reconnue
# sauf si l'ordre est correct dans les entrees dans le "case".
# 

ExecuteActionsFile ()
{
    RETURN_CODE=0
    



    DisplayTrace ""
    DisplayTrace ">>> ExecuteActionsFile"


    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
        
        CheckReadableFile ${1}
        RETURN_CODE=$?
    fi
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        FILE_ACTIONS=$1
        
        DisplayTrace ""
        DisplayTrace "    Executing actions from file :"
        DisplayTrace "    FILE_ACTIONS : \"${FILE_ACTIONS}\" "


        ${CAT} ${FILE_ACTIONS} | while read LINE
        do

            # DisplayTrace "LINE : \"${LINE}\" "
            DisplayTrace ""


            case ${LINE} in


            "INSTALL_FILES"*)

                DisplayTrace "INSTALL_FILES action ===================================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    InstallFiles "$2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "INSTALL_DIRECTORIES"*)

                DisplayTrace "INSTALL_DIRECTORIES action =============================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    InstallDirectories "$2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "UPDATE_FILE"*)

                DisplayTrace "UPDATE_FILE action =====================================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    UpdateFile "$2"
                    
                    RETURN_CODE=$?
                    
                    :
                fi 

                ;;


            "UPDATE_DIRECTORY"*)

                DisplayTrace "UPDATE_DIRECTORY action =============================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    UpdateDirectory "$2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "UPDATE_PLUGIN"*)

                DisplayTrace "UPDATE_PLUGIN action =============================================" 

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    UpdatePlugIn "$2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "")
                # echo "  Empty action \"${LINE}\" " 1>&2
                ;;   


            "#"*)
                # Commentaire
                
                ;;   



            *)
                DisplayTrace ""
                DisplayTrace "  Unknown action \"${LINE}\" " 
                DisplayTrace ""
                ;;   

            esac



           if [ ${RETURN_CODE} -ne 0 ]
           then   
                # Attention ! 
                # Le variable RETURN_CODE n'est pas modifiee en sortie car le 
                # "while" est execute dans un shell fils
                # Sa valeur en erreur n'est donc pas remontee a l'appelant
                 
                DisplayTrace ""
                DisplayTrace "    Error detected following action execution of \"${LINE}\"" 
                DisplayTrace "    RETURN_CODE : ${RETURN_CODE}"
                
                echo "" 1>&2
                echo "Error detected following action execution of \"${LINE}\" "  1>&2
                
                break
           fi       


        done
        
        
    fi

    return ${RETURN_CODE}
}



###########################################################################
# CheckFix
###########################################################################

# 
# 
#
#
CheckFix ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> CheckFix"

    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then

        # Controle version 
        
        CheckUpdatePlatform
        RETURN_CODE=$?
    fi


    if [ ${RETURN_CODE} -eq 0 ]
    then


        # Utilisation de la locale C pour etre sur d'obtenir la version
        # Sinon on a :
        # INZU417E COULD NOT DETERMINE CHARACTERS CODE PAGE FOR LOCALE fr_FR@euro (SPECIFY THIS LOCALE IN db2hpu.locale FILE)
        # INZU366I HPU RETURN CODE 8 (REASON CODE 0x111a001)

        VERSION_FULL=`(LANG=C ; ${LOCATION_PATH}/bin/db2hpu -v 2>&1) | ${AWK} '{print $NF}'`

        VERSION_BUILD_DATE=`${EXPR} "${VERSION_FULL}" : '.*(\(.*\))'`

        VERSION_NUMBER=`echo ${VERSION_FULL} | ${CUT} -d'(' -f1`

        DisplayTrace "    VERSION_FULL       : \"${VERSION_FULL}\" "
        DisplayTrace "    VERSION_BUILD_DATE : \"${VERSION_BUILD_DATE}\" "
        DisplayTrace "    VERSION_NUMBER     : \"${VERSION_NUMBER}\" "


    ###     VERSION_CURRENT_NUMERIC=`ConvertVersionToNumeric ${VERSION_NUMBER}`
    ###     VERSION_UPDATE_NUMERIC=`ConvertVersionToNumeric ${VERSION_UPDATE}`
    ###     VERSION_UPDATE_BASE_NUMERIC=`ConvertVersionToNumeric ${VERSION_UPDATE_BASE}`
    ###     
    ###     ### DEBUG ###
    ###     echo "Apres conversion numerique : "
    ###     echo "VERSION_CURRENT_NUMERIC : \"${VERSION_CURRENT_NUMERIC}\" "
    ###     echo "VERSION_UPDATE_NUMERIC  : \"${VERSION_UPDATE_NUMERIC}\" "
    ###     echo "VERSION_UPDATE_BASE_NUMERIC  : \"${VERSION_UPDATE_BASE_NUMERIC}\" "
    ###     ### FIN DEBUG ###


        CheckVersion ${VERSION_NUMBER} ${VERSION_UPDATE} ${VERSION_UPDATE_BASE}
        RETURN_CODE=$?
    fi
    

   
    return ${RETURN_CODE}
}



###########################################################################
# InstallUninstallDirectory
###########################################################################

# 
# Installation du script de desinstallation et des fichiers associes dans 
# l'arborescence HPU dans un sous-repertoire de HPU
# 
# Le sous-repertoire porte le nom de la mise-a-jour (variable UPDATE_LABEL)
# et se trouve dans 
# 
# Attention : liste de fichiers non parametree 
#

InstallUninstallDirectory ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> InstallUninstallDirectory"


    if [ -a ${LOCATION_PATH}/ptf ]
    then
        if [ -d ${LOCATION_PATH}/ptf ] 
        then
            if [ -a ${LOCATION_PATH}/ptf/${UPDATE_LABEL} ]
            then
                if [ -d ${LOCATION_PATH}/ptf/${UPDATE_LABEL} ]
                then
                    # Repertoire deja existant : ne rien signaler ?
                    :
                else
                    DisplayTrace "Error: ${LOCATION_PATH}/ptf/${UPDATE_LABEL} exists and is not a directory" 
                    echo "Error: ${LOCATION_PATH}/ptf/${UPDATE_LABEL} exists and is not a directory" 1>&2
                    RETURN_CODE=8
                fi
            else
                DisplayTrace "    Creating \"${LOCATION_PATH}/ptf/${UPDATE_LABEL}\" directory"
                
                ${MKDIR} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}
            fi
        else
           DisplayTrace "Error: ${LOCATION_PATH}/ptf/${UPDATE_LABEL} exists and is not a directory" 
           echo "Error: ${LOCATION_PATH}/ptf/${UPDATE_LABEL} exists and is not a directory" 1>&2
           RETURN_CODE=8
        fi
    else
        DisplayTrace "    Creating \"${LOCATION_PATH}/ptf\" directory"
        ${MKDIR} ${LOCATION_PATH}/ptf
        
        DisplayTrace "    Creating \"${LOCATION_PATH}/ptf/${UPDATE_LABEL}\" directory"
        ${MKDIR} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}
    fi
    
    
    DisplayTrace "    Populating files in \"${LOCATION_PATH}/ptf/${UPDATE_LABEL}\" directory"
    
    # Attention aux noms en cas de modifications des noms des fichiers
    
    ${CP} -p ${DIR_UPDATE}/uninstall_update_hpu.sh ${LOCATION_PATH}/ptf/${UPDATE_LABEL}
    
    # Recopie du script lui-meme
    ${CP} -p ${DIR_UPDATE}/${SCRIPT_NAME} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}

    ${CP} -p ${DIR_UPDATE}/${FILE_ACTIONS_INSTALL} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}
    
    ${CP} -p ${DIR_UPDATE}/${FILE_ACTIONS_PROLOG_INSTALL} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}
    ${CP} -p ${DIR_UPDATE}/${FILE_ACTIONS_EPILOG_INSTALL} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}
    ${CP} -p ${DIR_UPDATE}/${FILE_ACTIONS_PROLOG_UNINSTALL} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}
    ${CP} -p ${DIR_UPDATE}/${FILE_ACTIONS_EPILOG_UNINSTALL} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}

    ${CP} -p ${FILE_UPDATE_VARIABLES} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}
    ${CP} -p ${DIR_UPDATE}/${PLATFORM_CONFIG_FILE} ${LOCATION_PATH}/ptf/${UPDATE_LABEL}

    # Attention : nom explicite (a ameliorer)
    ${CP} -p ${DIR_UPDATE}/InstallInstructions.txt ${LOCATION_PATH}/ptf/${UPDATE_LABEL}

   
    return ${RETURN_CODE}
}



###########################################################################
# PreviewUpdate
###########################################################################

# 
# Previsualisation du traitement qui va etre effectue
# 
#
#
# 
#

PreviewUpdate ()
{
    RETURN_CODE=0


    DisplayTrace ""
    DisplayTrace ">>> PreviewUpdate"


    echo ""
    echo "The following actions will be taken:"
    # echo ""

    if [ ${RETURN_CODE} -eq 0 ]
    then
        case "${CONTEXT}" in
        "inst")
            PreviewUpdateInstallation
            ;;

        "uninst")
            PreviewUpdateUninstallation
            ;;

        *)
            echo "Unknown context \"${CONTEXT}\" " 1>&2
            RETURN_CODE=1
            ;;

        esac
    fi    
    
    return ${RETURN_CODE}
}



###########################################################################
# PreviewUpdateInstallation
###########################################################################

#
# Affichage des actions prevues (installation)
#
# 
# 
# - ExecutioXXXXn des actions du fichier prologue d'installation
# - Execution des actions des fichiers des "features"
# - Execution des actions du fichier epilogue d'installation
# 

PreviewUpdateInstallation ()
{
    RETURN_CODE=0

    echo ""
    DisplayTrace ">>> PreviewUpdateInstallation"


###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         echo "    Executing install prolog file"
###         ExecuteActionsFile "${FILE_ACTIONS_PROLOG_INSTALL}"
###         RETURN_CODE=$?
###     fi   

    if [ ${RETURN_CODE} -eq 0 ]
    then
        
        for FILE_ACTIONS in ${LIST_FILES_ACTIONS_INSTALL}
        do
            PreviewActionsFileInstallation "${FILE_ACTIONS}"
            RETURN_CODE=$?
        done    
    fi   
    
###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         echo "    Executing install epilog file"
###         ExecuteActionsFile "${FILE_ACTIONS_EPILOG_INSTALL}"
###         RETURN_CODE=$?
###     fi   



    if [ ${RETURN_CODE} -eq 0 ]
    then

        echo ""
        echo "All replaced files will be renamed by adding the \"_${UPDATE_LABEL}\" extention"
        echo ""
        echo "You must be the \"root\" user in order to run this update"
        echo ""
        echo "Type \"install\" to continue or \"quit\" to abort:"
        echo ""

        ANSWER=""
        CORRECT_ANSWER=0

        while [ ${CORRECT_ANSWER} -ne 1 ]
        do
            read ANSWER

            ANSWER=`echo "${ANSWER}" | ${TR} "[A-Z]" "[a-z]'`

            echo ""

            case "${ANSWER}" in
            "install")
                CORRECT_ANSWER=1
                ;;

            "quit")
                echo "Installation cancelled... No update done."
                CORRECT_ANSWER=1
                RETURN_CODE=10
                ;;

            *)
                echo "Please type \"install\" to continue or \"quit\" to abort"
                ;;
            esac
        done    
    fi

    echo ""
     
    return ${RETURN_CODE}
}


###########################################################################
# PreviewUpdateUninstallation
###########################################################################

#
# Affichage des actions prevues (desinstallation)
#
# 
# 
# - ExecutioXXXXn des actions du fichier prologue d'installation
# - Execution des actions des fichiers des "features"
# - Execution des actions du fichier epilogue d'installation
# 

PreviewUpdateUninstallation ()
{
    RETURN_CODE=0

    echo ""
    DisplayTrace ">>> PreviewUpdateUninstallation"

###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         echo "    Executing install prolog file"
###         ExecuteActionsFile "${FILE_ACTIONS_PROLOG_INSTALL}"
###         RETURN_CODE=$?
###     fi   

    if [ ${RETURN_CODE} -eq 0 ]
    then
        
        for FILE_ACTIONS in ${LIST_FILES_ACTIONS_UNINSTALL}
        do
            PreviewActionsFileUninstallation "${FILE_ACTIONS}"
            RETURN_CODE=$?
        done    
    fi   
    
###     if [ ${RETURN_CODE} -eq 0 ]
###     then
###         echo "    Executing install epilog file"
###         ExecuteActionsFile "${FILE_ACTIONS_EPILOG_INSTALL}"
###         RETURN_CODE=$?
###     fi   

    if [ ${RETURN_CODE} -eq 0 ]
    then

        echo ""
        echo "You must be the \"root\" user in order to run this update"
        echo ""
        echo "Type \"uninstall\" to continue or \"quit\" to abort:"
        echo ""
        echo ""

        ANSWER=""
        CORRECT_ANSWER=0

        while [ ${CORRECT_ANSWER} -ne 1 ]
        do
            read ANSWER

            ANSWER=`echo "${ANSWER}" | ${TR} "[A-Z]" "[a-z]'`

            echo ""

            case "${ANSWER}" in
            "uninstall")
                CORRECT_ANSWER=1
                ;;

            "quit")
                echo "Uninstallation cancelled... No update done."
                CORRECT_ANSWER=1
                RETURN_CODE=10
                ;;

            *)
                echo "Please type \"uninstall\" to continue or \"quit\" to abort"
                ;;
            esac
        done    
    fi

    echo ""

    return ${RETURN_CODE}
}


###########################################################################
# PreviewActionsFileInstallation
###########################################################################

#
# Affichage des directives contenues dans un fichier d'actions (installation)
#
# 
# Argument 1 : fichier des actions a executer
# 
# 
# Attention : les noms des commandes ne doivent pas debuter par une meme
# portion de texte sinon une commande risque de ne jamais etre reconnue
# sauf si l'ordre est correct dans les entrees dans le "case".
# 

PreviewActionsFileInstallation ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> PreviewActionsFileInstallation"


    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
        
        CheckReadableFile ${1}
        RETURN_CODE=$?
    fi
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        FILE_ACTIONS=$1
        
        DisplayTrace ""
        DisplayTrace "    Executing actions from file :"
        DisplayTrace "    FILE_ACTIONS : \"${FILE_ACTIONS}\" "


        ${CAT} ${FILE_ACTIONS} | while read LINE
        do

            # DisplayTrace "LINE : \"${LINE}\" "
            DisplayTrace ""

            case ${LINE} in

            "INSTALL_FILES"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "- Install files $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;

            "INSTALL_DIRECTORIES"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "- Install directories $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "UPDATE_FILE"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "Update file $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "UPDATE_DIRECTORY"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "- Update directory $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "UPDATE_PLUGIN"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "- Update plug-in $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;



            "")
                # echo "  Empty action \"${LINE}\" " 1>&2
                ;;   


            "#"*)
                # Commentaire
                
                ;;   



            *)
                DisplayTrace ""
                DisplayTrace "  Unknown action \"${LINE}\" " 
                DisplayTrace ""
                ;;   

            esac

        done
        
        
    fi

    return ${RETURN_CODE}
}


###########################################################################
# PreviewActionsFileUninstallation
###########################################################################

#
# Affichage des directives contenues dans un fichier d'actions (desinstallation)
#
# 
# Argument 1 : fichier des actions a executer
# 
# 
# Attention : les noms des commandes ne doivent pas debuter par une meme
# portion de texte sinon une commande risque de ne jamais etre reconnue
# sauf si l'ordre est correct dans les entrees dans le "case".
# 

PreviewActionsFileUninstallation ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> PreviewActionsFileUninstallation"


    if [ $# -ne 1 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
        
        CheckReadableFile ${1}
        RETURN_CODE=$?
    fi
    
    
    if [ ${RETURN_CODE} -eq 0 ]
    then
        FILE_ACTIONS=$1
        
        DisplayTrace ""
        DisplayTrace "    Executing actions from file :"
        DisplayTrace "    FILE_ACTIONS : \"${FILE_ACTIONS}\" "


        ${CAT} ${FILE_ACTIONS} | while read LINE
        do

            # DisplayTrace "LINE : \"${LINE}\" "
            DisplayTrace ""

            case ${LINE} in

            "INSTALL_FILES"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "- Install files $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "INSTALL_DIRECTORIES"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "- Install directories $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "UPDATE_FILE"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "Restore file $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "UPDATE_DIRECTORY"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "- Restore directory $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "UPDATE_PLUGIN"*)

                \set ${LINE}
                if [ $# -ne 2 ]
                then
                    echo "Missing argument for action \"$1\"" 1>&2
                    RETURN_CODE=1
                    break
                else
                    echo "- Restore plug-in $2"
                    
                    RETURN_CODE=$?
                fi 

                ;;


            "")
                # echo "  Empty action \"${LINE}\" " 1>&2
                ;;   


            "#"*)
                # Commentaire
                
                ;;   


            *)
                DisplayTrace ""
                DisplayTrace "  Unknown action \"${LINE}\" " 
                DisplayTrace ""
                ;;   

            esac

        done
        
        
    fi

    return ${RETURN_CODE}
}


###########################################################################
# ListInstalledUpdates
###########################################################################

#
# Affichage de la liste des mises-a-jour deja installees
#
# 
# Argument 1 : fichier des actions a executer
# 
# 
# Methode inutilisee : la liste est effectuee en listant les sous-repertoires du
# repertoire contenant les mises-a-jour deja installees
# 
# Methode utilisee : liste des fichiers executables db2hpum_debug_XXXX
# Pb : les noms sont des noms de abreges pour les noms de fichiers et ne sont pas
# les noms officiels des mises-a-jour.  
# 

ListInstalledUpdates ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> ListInstalledUpdates"

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    
        LIST_INSTALLED_UPDATES=""

        if [ -d ${LOCATION_PATH}/ptf ]
        then
            ${LS} -1 ${LOCATION_PATH}/bin/db2hpum_debug_* >/dev/null 2>&1
            if [ $? -eq 0 ]
            then

                ${LS} -1 ${LOCATION_PATH}/bin/db2hpum_debug_* | while read NAME
                do
                    NAME=`\basename ${NAME}`

                    TMP_UPDATE_LABEL=`${EXPR} "${NAME}" ":" "db2hpum_debug_"'\(.*\)$'`
                    LIST_INSTALLED_UPDATES="${LIST_INSTALLED_UPDATES} ${TMP_UPDATE_LABEL}"
                done

            fi
        fi
        
        echo "Installed updates :"
        if [ -z "${LIST_INSTALLED_UPDATES}" ]
        then
            echo "None"
        else    
            echo "${LIST_INSTALLED_UPDATES}"

        fi        
    fi    

    return ${RETURN_CODE}
}


###########################################################################
# DisplayUpdateInformation
###########################################################################

#
# Affichage des informations sur la mise-a-jour
#
# 
# 
# 
# 
# 
# 

DisplayUpdateInformation ()
{
    RETURN_CODE=0
    

    DisplayTrace ""
    DisplayTrace ">>> DisplayUpdateInformation"

    if [ $# -ne 0 ]
    then
        echo "Missing or too many arguments ($#)" 1>&2
        RETURN_CODE=1
    else    

        echo ""
        echo "-------------------------------------------------------------------"
        echo ""
        echo "High Performance Unload"
        echo ""
        echo "${UPDATE_CATEGORY} ${UPDATE_LABEL} for `UnformatPlatformName ${UPDATE_OS}` operating system"
        echo ""
        echo ""
        echo "-------------------------------------------------------------------"
        echo ""
        
        case "${CONTEXT}" in
        "inst")
            DisplayTrace "    The update will be applied to the version located at"
            echo "The update will be applied to the version located at"
            ;;

        "uninst")
            DisplayTrace "    The update will be removed from the version located at"
            echo "The update will be removed from the version located at"
            ;;

        *)
            echo "Unknown context \"${CONTEXT}\" " 1>&2
            RETURN_CODE=1
            ;;
        esac    

        DisplayTrace "    ${LOCATION_PATH}"
        echo "${LOCATION_PATH}"
        
        echo ""
        echo ""

        # Liste des mises-a-jour deja installees
        # Desactive actuellement, car une mise-a-jour desinstallee n'est pas 
        # distinguable d'une mise-a-jour encore installe (repertoire tuojours
        # present

        if [ ${RETURN_CODE} -eq 0 ]
        then
            ListInstalledUpdates
        fi

    fi    

    return ${RETURN_CODE}
}




###########################################################################
# main
###########################################################################

# 
# Programme principal (point d'entree du script)
#
# A faire : 
# Gestion fichier journal (log)
# Actuellement les fichiers journaux sont geres par InstallShield 
# dans le "bean" executant le script d'installation
#
# Argument 1 : contexte d'utilisation ("inst" ou "uninst")
# Argument 2 : repertoire d'installation actuel de HPU
# ## Argument 1 : contexte d'utilisation ("inst" ou "uninst")
# ## Argument 2 : chemin complet vers le fichier contenant les variables InstallShield
#
#

RETURN_CODE=0

### \echo "Number of arguments : $#"
### DisplayTrace "Arguments           : $*"
### \echo "Current directory   : `pwd`"

SCRIPT_NAME=`\basename $0`

# Obtention du chemin complet du repertoire ou est installe le HotFix
# DIR_UPDATE a le meme role que DIRBASE (installation HotFix 2.1.x)
# A faire d'abord car on a besoin pour le controle des arguments d'appel

DIR_UPDATE=`\dirname $0`
if [ `\echo ${DIR_UPDATE} | \cut -c 1` = "." ]
then
    DIR_UPDATE=`\pwd``\echo ${DIR_UPDATE} | cut -c 2-`
else
    if [ `\echo ${DIR_UPDATE} | \cut -c 1` != "/" ]
    then
        DIR_UPDATE=`\pwd`/${DIR_UPDATE}
    fi
fi


### echo "DIR_UPDATE : \"${DIR_UPDATE}"


if [ $# -eq 0 ]
then
    echo "Incorrect number or arguments" 1>&2
    exit 1
fi







# Analyse du premier argument d'abord car nombre variable d'arguments



CONTEXT="$1"

# Controle sur le contexte d'utilisation
# Initialisation des fichies de trace

CONTEXT=`echo "${CONTEXT}" | \tr '[A-Z]' '[a-z]'`
if [ ${RETURN_CODE} -eq 0 ]
then
    case "${CONTEXT}" in
    "inst")
        # Fichier de trace
        FILE_TRACE="${DIR_UPDATE}/trace_install_update_hpu"
        \rm -f ${FILE_TRACE}
    
        DisplayTrace "Context      : Installation"
        ;;

    "uninst")
        # Fichier de trace
        FILE_TRACE="${DIR_UPDATE}/trace_uninstall_update_hpu"
        \rm -f ${FILE_TRACE}
        
        DisplayTrace "Context      : Uninstallation"
        ;;

    *)
        echo "Unknown context \"${CONTEXT}\" " 1>&2
        RETURN_CODE=1
        ;;

    esac
fi








# Controle sur le second argument 

if [ ${RETURN_CODE} -eq 0 ]
then
    case "${CONTEXT}" in
    "inst")
        
        # Deuxieme argument obligatoire
        
        if [ $# -ne 2 ]
        then
            echo "Incorrect number or arguments" 1>&2
            RETURN_CODE=1
        else

            LOCATION_PATH=$2
        fi

        ;;

    "uninst")
    
        # Pas de deuxieme argument 
        if [ $# -ne 1 ]
        then
            echo "Incorrect number or arguments" 1>&2
            RETURN_CODE=1
        else

            # Le repertoire de desinstallation se trouve sous deux niveaux de repertoire
            
            DIR_CURRENT=`\pwd`
            
            \cd ${DIR_UPDATE}/../.. 
            if [ $? -eq 0 ]
            then
                LOCATION_PATH=`\pwd`
            else
                DisplayTrace "    Cannot locate HPU installation directory"
                DisplayTrace "    Unexpected location for uninstallation script and files"

                echo "    Cannot locate HPU installation directory" 1>&2
                echo "    Unexpected location for uninstallation script and files" 1>&2
                RETURN_CODE=1
            fi
            
            \cd ${DIR_CURRENT}  
            
        fi        
        ;;

    *)
        echo "Unknown context \"${CONTEXT}\" " 1>&2
        RETURN_CODE=1
        ;;

    esac
fi


# Initialisation de variables globales avec des valeurs par defaut (AWK, ...)
# (A faire le plut tot possible)

if [ ${RETURN_CODE} -eq 0 ]
then
    InitDefaultVariables
    RETURN_CODE=$?
fi



# Repertoire d'installation de HPU

if [ ${RETURN_CODE} -eq 0 ]
then
    DisplayTrace ""
    DisplayTrace "LOCATION_PATH : \"${LOCATION_PATH}\" "

    CheckExistingDirectory ${LOCATION_PATH}
    RETURN_CODE=$?
fi


# Controle identite utilisateur


if [ ${RETURN_CODE} -eq 0 ]
then
    USER_ID=`GetUserID` 
    
    if [ "${USER_ID}" -eq 0 ]
    then
        DisplayTrace ""
        DisplayTrace "    Current UID is \"root\"."
    else
        DisplayTrace ""
        DisplayTrace "    Current UID is not \"root\" : ${USER_ID}"
        DisplayTrace "    Aborting. "
        
        echo "    You are not the \"root\" user." 1>&2
        echo "    You must be the \"root\" user in order to run this update." 1>&2
        echo "    Aborting. " 1>&2

        RETURN_CODE=1
    fi    
fi




# Lecture du fichier de variables du hotfix


# Nom de fichier a parametrer ?

if [ ${RETURN_CODE} -eq 0 ]
then
    # Recherche fichier des variables de la mise-a-jour
    FILE_UPDATE_VARIABLES="${DIR_UPDATE}/update_variables"

    CheckReadableFile "${FILE_UPDATE_VARIABLES}"
    RETURN_CODE=$?


fi


# Initialisation des variables shell incluses dans le fichier 
# des variables hotfix
# Ajouter controle si variables existantes apres...

# Rq : LOCATION_PATH :(deuxieme argument du script) est reinitialise avec  la
#      meme valeur normalement

if [ ${RETURN_CODE} -eq 0 ]
then
    . ${FILE_UPDATE_VARIABLES}


    DisplayTrace ""
    DisplayTrace "    Data from HotFix : " 
    DisplayTrace "    (file \"${FILE_UPDATE_VARIABLES}\")"
    DisplayTrace ""
    DisplayTrace "    UPDATE_LABEL        : \"${UPDATE_LABEL}\" "
    DisplayTrace "    UPDATE_TYPE         : \"${UPDATE_TYPE}\" "
    DisplayTrace "    VERSION_UPDATE      : \"${VERSION_UPDATE}\" "
    DisplayTrace "    VERSION_UPDATE_BASE : \"${VERSION_UPDATE_BASE}\" "
    DisplayTrace "    UPDATE_OS           : \"${UPDATE_OS}\" "
  
  
    case "${UPDATE_TYPE}" in
    "INTERIM_FIX")
        UPDATE_CATEGORY="Interim Fix"
        ;;
        
   "HOT_FIX")
        UPDATE_CATEGORY="Hot Fix"
        ;;
   
   *)     
        DisplayTrace "" 
        DisplayTrace "    Unknown UPDATE_TYPE value : \"${UPDATE_TYPE}\" "
        UPDATE_CATEGORY="Update"
        ;;
   esac     
    
fi





# Lecture du fichier de variables de l'installation actuelle
# Nom de fichier a parametrer ?

if [ ${RETURN_CODE} -eq 0 ]
then

    if [ -d "${LOCATION_PATH}/install" ]
    then
        # Recherche fichier is_variables
        FILE_IS_VARIABLES="${LOCATION_PATH}/install/is_variables"

        CheckReadableFile "${FILE_IS_VARIABLES}"
        RETURN_CODE=$?

    fi
fi




# Initialisation des variables shell incluses dans le fichier provenant 
# de l'installation actuelle (InstallShield)
# Ajouter controle si variables existantes apres...

if [ ${RETURN_CODE} -eq 0 ]
then
    . ${FILE_IS_VARIABLES}


    DisplayTrace ""
    DisplayTrace "    Data transmitted from InstallShield : " 
    DisplayTrace "    (file \"${FILE_IS_VARIABLES}\")"
    DisplayTrace ""
    DisplayTrace "    FEATURES         : \"${FEATURES}\" "
    DisplayTrace "    LOCATION_PATH    : \"${LOCATION_PATH}\" "
    DisplayTrace "    LOG              : \"${LOG}\" "
    DisplayTrace "    DEFAULT_DATABASE : \"${DEFAULT_DATABASE}\" "
    DisplayTrace "    DEFAULT_INSTANCE : \"${DEFAULT_INSTANCE}\" "
    DisplayTrace "    DEFAULT_HOME     : \"${DEFAULT_HOME}\" "
    DisplayTrace "    PLATFORM         : \"${PLATFORM}\" "
    DisplayTrace "    ARCHITECTURE     : \"${ARCHITECTURE}\" "
    DisplayTrace "    VERSION          : \"${VERSION}\" "
    
    DisplayTrace "    TEMPORARY_PATH   : \"${TEMPORARY_PATH}\" "
    
fi


if [ ${RETURN_CODE} -eq 0 ]
then
    DisplayUpdateInformation
    
fi




# Controles sur l'applicabilite de la mise-a-jour
if [ ${RETURN_CODE} -eq 0 ]
then
    if [ "${CONTEXT}" = "inst" ]
    then
        CheckFix
        RETURN_CODE=$?
    fi    
fi



if [ ${RETURN_CODE} -eq 0 ]
then
    \cd ${DIR_UPDATE}
fi


### DEBUG ###
### # Suppression fichiers temporaires resultant de l'execution precedente
### \rm -f ${DIR_UPDATE}/temp/tmp_section_* 
### \rm -f ${DIR_UPDATE}/temp/tmp_install_* 
### \rm -Rf ${DIR_UPDATE}/temp
### FIN DEBUG ###



if [ ${RETURN_CODE} -eq 0 ]
then
    InitVariables
    RETURN_CODE=$?
fi


if [ ${RETURN_CODE} -eq 0 ]
then
    CheckEnvironment
    RETURN_CODE=$?
fi   





if [ ${RETURN_CODE} -eq 0 ]
then
    PrepareActionsLists
    RETURN_CODE=$?
fi



# Presentation de ce qui va etre effectue et demande de confirmation
if [ ${RETURN_CODE} -eq 0 ]
then
    PreviewUpdate
    RETURN_CODE=$?
fi



if [ ${RETURN_CODE} -eq 0 ]
then
    ExtractArchive
    RETURN_CODE=$?
fi



# Execution des actions selon le mode d'utilisation

if [ ${RETURN_CODE} -eq 0 ]
then
    ExecuteActions
    RETURN_CODE=$?
fi



# Installation du script de desinstallation dans l'arborescence HPU

if [ ${RETURN_CODE} -eq 0 ]
then
    if [ "${CONTEXT}" = "inst" ]
    then
        InstallUninstallDirectory
        RETURN_CODE=$?
    fi    
fi






 
# Suppression des fichiers temporaires (sections, ...)

DeleteTemporaryFiles


if [ ${RETURN_CODE} -eq 0 ]
then
    DisplayTrace ""
    echo ""

    case "${CONTEXT}" in
    "inst")
        DisplayTrace "Installation completed."
        echo "Installation completed."
        ;;

    "uninst")
        DisplayTrace "Uninstallation completed."
        echo "Uninstallation completed."
        ;;

    *)
        echo "Unknown context \"${CONTEXT}\" " 1>&2
        RETURN_CODE=1
        ;;

    esac
fi





exit ${RETURN_CODE}

